"""Generated message classes for file version v1p1alpha1.

The Cloud Filestore API is used for creating and managing cloud file servers.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'file'


class Backup(_messages.Message):
  r"""A Filestore backup.

  Enums:
    SourceInstanceTierValueValuesEnum: Output only. The service tier of the
      source Filestore instance that this backup is created from.
    StateValueValuesEnum: Output only. The backup state.

  Messages:
    LabelsValue: Resource labels to represent user provided metadata.

  Fields:
    capacityGb: Output only. Capacity of the source file share when the backup
      was created.
    createTime: Output only. The time when the backup was created.
    description: A description of the backup with 2048 characters or less.
      Requests with longer descriptions will be rejected.
    downloadBytes: Output only. Amount of bytes that will be downloaded if the
      backup is restored
    labels: Resource labels to represent user provided metadata.
    name: Output only. The resource name of the backup, in the format
      `projects/{project_id}/locations/{location_id}/backups/{backup_id}`.
    satisfiesPzi: Output only. Reserved for future use.
    satisfiesPzs: Output only. Reserved for future use.
    sourceFileShare: Name of the file share in the source Filestore instance
      that the backup is created from.
    sourceInstance: The resource name of the source Filestore instance, in the
      format
      `projects/{project_id}/locations/{location_id}/instances/{instance_id}`,
      used to create this backup.
    sourceInstanceTier: Output only. The service tier of the source Filestore
      instance that this backup is created from.
    state: Output only. The backup state.
    storageBytes: Output only. The size of the storage used by the backup. As
      backups share storage, this number is expected to change with backup
      creation/deletion.
  """

  class SourceInstanceTierValueValuesEnum(_messages.Enum):
    r"""Output only. The service tier of the source Filestore instance that
    this backup is created from.

    Values:
      TIER_UNSPECIFIED: Not set.
      STANDARD: STANDARD tier. BASIC_HDD is the preferred term for this tier.
      PREMIUM: PREMIUM tier. BASIC_SSD is the preferred term for this tier.
      BASIC_HDD: BASIC instances offer a maximum capacity of 63.9 TB.
        BASIC_HDD is an alias for STANDARD Tier, offering economical
        performance backed by HDD.
      BASIC_SSD: BASIC instances offer a maximum capacity of 63.9 TB.
        BASIC_SSD is an alias for PREMIUM Tier, and offers improved
        performance backed by SSD.
      HIGH_SCALE_SSD: HIGH_SCALE instances offer expanded capacity and
        performance scaling capabilities.
      ENTERPRISE: ENTERPRISE instances offer the features and availability
        needed for mission-critical workloads.
      ZONAL: ZONAL instances offer large/small capacities and performance
        scaling capabilities.
      REGIONAL: REGIONAL instances offer the features and availability needed
        for mission-critical workloads.
    """
    TIER_UNSPECIFIED = 0
    STANDARD = 1
    PREMIUM = 2
    BASIC_HDD = 3
    BASIC_SSD = 4
    HIGH_SCALE_SSD = 5
    ENTERPRISE = 6
    ZONAL = 7
    REGIONAL = 8

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The backup state.

    Values:
      STATE_UNSPECIFIED: State not set.
      CREATING: Backup is being created.
      FINALIZING: Backup has been taken and the operation is being finalized.
        At this point, changes to the file share will not be reflected in the
        backup.
      READY: Backup is available for use.
      DELETING: Backup is being deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    FINALIZING = 2
    READY = 3
    DELETING = 4

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Resource labels to represent user provided metadata.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  capacityGb = _messages.IntegerField(1)
  createTime = _messages.StringField(2)
  description = _messages.StringField(3)
  downloadBytes = _messages.IntegerField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  satisfiesPzi = _messages.BooleanField(7)
  satisfiesPzs = _messages.BooleanField(8)
  sourceFileShare = _messages.StringField(9)
  sourceInstance = _messages.StringField(10)
  sourceInstanceTier = _messages.EnumField('SourceInstanceTierValueValuesEnum', 11)
  state = _messages.EnumField('StateValueValuesEnum', 12)
  storageBytes = _messages.IntegerField(13)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class CopyInstanceRequest(_messages.Message):
  r"""CopyInstanceRequest copies a fileshare from a Basic instance to an
  Enterprise or High Scale instance.

  Fields:
    reservedIpRange: The IP address range in the VPC of the source and target
      instances that can be used for networking resources that need to be
      created by the service for the subnet needed during the operation to
      perform the copy. If the instance uses IP-based access control, the
      source instance must allow reads from the IP range and the target
      instance must allow writes from the IP range. It must be a /29 CIDR
      block in the internal IP address ranges
      (https://www.arin.net/knowledge/address_filters.html). For example,
      10.0.0.0/29.
    sourceFileShare: Required. Name of the source file share in the Filestore
      Basic instance that we are copying the data from.
    sourceInstance: Required. The name of the Basic instance that we are
      copying the fileshare from, in the format `projects/{project_number}/loc
      ations/{location}/instances/{instance_id}`.
    targetFileShare: Required. Name of the target file share in the Filestore
      High Scale or Enterprise instance that we are copying the data to.
  """

  reservedIpRange = _messages.StringField(1)
  sourceFileShare = _messages.StringField(2)
  sourceInstance = _messages.StringField(3)
  targetFileShare = _messages.StringField(4)


class DailyCycle(_messages.Message):
  r"""Time window specified for daily operations.

  Fields:
    duration: Output only. Duration of the time window, set by service
      producer.
    startTime: Time within the day to start the operations.
  """

  duration = _messages.StringField(1)
  startTime = _messages.MessageField('TimeOfDay', 2)


class Date(_messages.Message):
  r"""Represents a whole or partial calendar date, such as a birthday. The
  time of day and time zone are either specified elsewhere or are
  insignificant. The date is relative to the Gregorian Calendar. This can
  represent one of the following: * A full date, with non-zero year, month,
  and day values. * A month and day, with a zero year (for example, an
  anniversary). * A year on its own, with a zero month and a zero day. * A
  year and month, with a zero day (for example, a credit card expiration
  date). Related types: * google.type.TimeOfDay * google.type.DateTime *
  google.protobuf.Timestamp

  Fields:
    day: Day of a month. Must be from 1 to 31 and valid for the year and
      month, or 0 to specify a year by itself or a year and month where the
      day isn't significant.
    month: Month of a year. Must be from 1 to 12, or 0 to specify a year
      without a month and day.
    year: Year of the date. Must be from 1 to 9999, or 0 to specify a date
      without a year.
  """

  day = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  month = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  year = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class DenyMaintenancePeriod(_messages.Message):
  r"""DenyMaintenancePeriod definition. Maintenance is forbidden within the
  deny period. The start_date must be less than the end_date.

  Fields:
    endDate: Deny period end date. This can be: * A full date, with non-zero
      year, month and day values. * A month and day value, with a zero year.
      Allows recurring deny periods each year. Date matching this period will
      have to be before the end.
    startDate: Deny period start date. This can be: * A full date, with non-
      zero year, month and day values. * A month and day value, with a zero
      year. Allows recurring deny periods each year. Date matching this period
      will have to be the same or after the start.
    time: Time in UTC when the Blackout period starts on start_date and ends
      on end_date. This can be: * Full time. * All zeros for 00:00:00 UTC
  """

  endDate = _messages.MessageField('Date', 1)
  startDate = _messages.MessageField('Date', 2)
  time = _messages.MessageField('TimeOfDay', 3)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class ExportConfig(_messages.Message):
  r"""File share export specification.

  Fields:
    allowedIpRanges: List of IPv4 addresses in the format
      `{octet1}.{octet2}.{octet3}.{octet4}` or CIDR ranges in the format
      `{octet1}.{octet2}.{octet3}.{octet4}/{mask size}` which may mount the
      fileshare.
  """

  allowedIpRanges = _messages.StringField(1, repeated=True)


class FileProjectsLocationsBackupsCreateRequest(_messages.Message):
  r"""A FileProjectsLocationsBackupsCreateRequest object.

  Fields:
    backup: A Backup resource to be passed as the request body.
    backupId: Required. The ID to use for the backup. The ID must be unique
      within the specified project and location. This value must start with a
      lowercase letter followed by up to 62 lowercase letters, numbers, or
      hyphens, and cannot end with a hyphen.
    parent: Required. The backup's project and location, in the format
      `projects/{project_number}/locations/{location}`. In Filestore, backup
      locations map to GCP regions, for example **us-west1**.
  """

  backup = _messages.MessageField('Backup', 1)
  backupId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class FileProjectsLocationsBackupsDeleteRequest(_messages.Message):
  r"""A FileProjectsLocationsBackupsDeleteRequest object.

  Fields:
    name: Required. The backup resource name, in the format
      `projects/{project_number}/locations/{location}/backups/{backup_id}`
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsBackupsGetRequest(_messages.Message):
  r"""A FileProjectsLocationsBackupsGetRequest object.

  Fields:
    name: Required. The backup resource name, in the format
      `projects/{project_number}/locations/{location}/backups/{backup_id}`.
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsBackupsListRequest(_messages.Message):
  r"""A FileProjectsLocationsBackupsListRequest object.

  Fields:
    filter: List filter.
    orderBy: Sort results. Supported values are "name", "name desc" or ""
      (unsorted).
    pageSize: The maximum number of items to return.
    pageToken: The next_page_token value to use if there are additional
      results to retrieve for this list request.
    parent: Required. The project and location for which to retrieve backup
      information, in the format
      `projects/{project_number}/locations/{location}`. In Filestore, backup
      locations map to GCP regions, for example **us-west1**. To retrieve
      backup information for all locations, use "-" for the `{location}`
      value.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class FileProjectsLocationsBackupsPatchRequest(_messages.Message):
  r"""A FileProjectsLocationsBackupsPatchRequest object.

  Fields:
    backup: A Backup resource to be passed as the request body.
    name: Output only. The resource name of the backup, in the format
      `projects/{project_id}/locations/{location_id}/backups/{backup_id}`.
    updateMask: Required. Mask of fields to update. At least one path must be
      supplied in this field.
  """

  backup = _messages.MessageField('Backup', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class FileProjectsLocationsGetRequest(_messages.Message):
  r"""A FileProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsInstancesCopyInstanceRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesCopyInstanceRequest object.

  Fields:
    copyInstanceRequest: A CopyInstanceRequest resource to be passed as the
      request body.
    targetInstance: Required. The name of the Enterprise or High Scale
      instance instance that we are copying fileshare to, in the format `proje
      cts/{project_number}/locations/{location}/instances/{instance_id}`.
  """

  copyInstanceRequest = _messages.MessageField('CopyInstanceRequest', 1)
  targetInstance = _messages.StringField(2, required=True)


class FileProjectsLocationsInstancesCreateRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesCreateRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    instanceId: Required. The ID of the instance to create. The ID must be
      unique within the specified project and location. This value must start
      with a lowercase letter followed by up to 62 lowercase letters, numbers,
      or hyphens, and cannot end with a hyphen.
    parent: Required. The instance's project and location, in the format
      `projects/{project_number}/locations/{location}`. In Filestore,
      locations map to GCP zones, for example **us-west1-b**.
  """

  instance = _messages.MessageField('Instance', 1)
  instanceId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class FileProjectsLocationsInstancesDeleteRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesDeleteRequest object.

  Fields:
    name: Required. The instance resource name, in the format
      `projects/{project_number}/locations/{location}/instances/{instance_id}`
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsInstancesGetRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesGetRequest object.

  Fields:
    name: Required. The instance resource name, in the format
      `projects/{project_number}/locations/{location}/instances/{instance_id}`
      .
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsInstancesListRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesListRequest object.

  Fields:
    filter: List filter.
    orderBy: Sort results. Supported values are "name", "name desc" or ""
      (unsorted).
    pageSize: The maximum number of items to return.
    pageToken: The next_page_token value to use if there are additional
      results to retrieve for this list request.
    parent: Required. The project and location for which to retrieve instance
      information, in the format
      `projects/{project_number}/locations/{location}`. In Cloud Filestore,
      locations map to GCP zones, for example **us-west1-b**. To retrieve
      instance information for all locations, use "-" for the `{location}`
      value.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class FileProjectsLocationsInstancesPatchRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesPatchRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    name: Output only. The resource name of the instance, in the format `proje
      cts/{project_number}/locations/{location_id}/instances/{instance_id}`.
    updateMask: Required. Mask of fields to update. At least one path must be
      supplied in this field. The elements of the repeated paths field may
      only include these fields: * "description" * "file_shares" * "labels"
  """

  instance = _messages.MessageField('Instance', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class FileProjectsLocationsInstancesRestoreRequest(_messages.Message):
  r"""A FileProjectsLocationsInstancesRestoreRequest object.

  Fields:
    name: Required. The resource name of the instance, in the format
      `projects/{project_id}/locations/{location_id}/instances/{instance_id}`.
    restoreInstanceRequest: A RestoreInstanceRequest resource to be passed as
      the request body.
  """

  name = _messages.StringField(1, required=True)
  restoreInstanceRequest = _messages.MessageField('RestoreInstanceRequest', 2)


class FileProjectsLocationsListRequest(_messages.Message):
  r"""A FileProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class FileProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A FileProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class FileProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A FileProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A FileProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A FileProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class FileProjectsLocationsSnapshotsCreateRequest(_messages.Message):
  r"""A FileProjectsLocationsSnapshotsCreateRequest object.

  Fields:
    parent: Required. The snapshot's project and location, in the format
      `projects/{project_number}/locations/{location}`. In Filestore, snapshot
      locations map to GCP zones, for example **us-west1-b**, for local
      snapshots and to GCP regions, for example **us-west1**, otherwise.
    snapshot: A Snapshot resource to be passed as the request body.
    snapshotId: Required. The ID to use for the snapshot. The ID must be
      unique within the specified project and location. This value must start
      with a lowercase letter followed by up to 62 lowercase letters, numbers,
      or hyphens, and cannot end with a hyphen.
  """

  parent = _messages.StringField(1, required=True)
  snapshot = _messages.MessageField('Snapshot', 2)
  snapshotId = _messages.StringField(3)


class FileProjectsLocationsSnapshotsDeleteRequest(_messages.Message):
  r"""A FileProjectsLocationsSnapshotsDeleteRequest object.

  Fields:
    name: Required. The snapshot resource name, in the format
      `projects/{project_number}/locations/{location}/snapshots/{snapshot_id}`
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsSnapshotsGetRequest(_messages.Message):
  r"""A FileProjectsLocationsSnapshotsGetRequest object.

  Fields:
    name: Required. The snapshot resource name, in the format
      `projects/{project_number}/locations/{location}/snapshots/{snapshot_id}`
      .
  """

  name = _messages.StringField(1, required=True)


class FileProjectsLocationsSnapshotsListRequest(_messages.Message):
  r"""A FileProjectsLocationsSnapshotsListRequest object.

  Fields:
    filter: List filter.
    orderBy: Sort results. Supported values are "name", "name desc" or ""
      (unsorted).
    pageSize: The maximum number of items to return.
    pageToken: The next_page_token value to use if there are additional
      results to retrieve for this list request.
    parent: Required. The project and location for which to retrieve snapshot
      information, in the format
      `projects/{project_number}/locations/{location}`. In Filestore, snapshot
      locations map to GCP zones, for example **us-west1-b**, for local
      snapshots and to GCP regions, for example **us-west1**, otherwise. To
      retrieve snapshot information for all locations, use "-" for the
      `{location}` value.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class FileProjectsLocationsSnapshotsPatchRequest(_messages.Message):
  r"""A FileProjectsLocationsSnapshotsPatchRequest object.

  Fields:
    name: Output only. The resource name of the snapshot, in the format
      `projects/{project_id}/locations/{location_id}/snapshots/{snapshot_id}`.
    snapshot: A Snapshot resource to be passed as the request body.
    updateMask: Required. Mask of fields to update. At least one path must be
      supplied in this field.
  """

  name = _messages.StringField(1, required=True)
  snapshot = _messages.MessageField('Snapshot', 2)
  updateMask = _messages.StringField(3)


class FileShareConfig(_messages.Message):
  r"""File share configuration for the instance.

  Fields:
    capacityGb: File share capacity in gigabytes (GB). Filestore defines 1 GB
      as 1024^3 bytes.
    exports: Exports. If exports are omitted, a default allowed_ip_ranges is
      set to RFC1918 Internal IP ranges (10.0.0.0/8, 172.16.0.0/12,
      192.168.0.0/16) This field is deprecated, use nfs_export_options
      instead.
    name: Required. The name of the file share. Must use 1-16 characters for
      the basic service tier and 1-63 characters for all other service tiers.
      Must use lowercase letters, numbers, or underscores [a-z0-9_]. Must
      start with a letter. Immutable.
    nfsExportOptions: Nfs Export Options. There is a limit of 10 export
      options per file share.
    sourceBackup: The resource name of the backup, in the format
      `projects/{project_id}/locations/{location_id}/backups/{backup_id}`,
      that this file share has been restored from. Empty, if the file share is
      created from scratch and not restored from a backup.
    sourceSnapshot: The resource name of the snapshot, in the format
      `projects/{project_id}/locations/{location_id}/snapshots/{snapshot_id}`,
      that this file share has been restored from. Empty, if the file share is
      created from scratch and not restored from a snapshot.
  """

  capacityGb = _messages.IntegerField(1)
  exports = _messages.MessageField('ExportConfig', 2, repeated=True)
  name = _messages.StringField(3)
  nfsExportOptions = _messages.MessageField('NfsExportOptions', 4, repeated=True)
  sourceBackup = _messages.StringField(5)
  sourceSnapshot = _messages.StringField(6)


class GoogleCloudSaasacceleratorManagementProvidersV1Instance(_messages.Message):
  r"""Instance represents the interface for SLM services to actuate the state
  of control plane resources. Example Instance in JSON, where consumer-
  project-number=123456, producer-project-id=cloud-sql: ```json Instance: {
  "name": "projects/123456/locations/us-east1/instances/prod-instance",
  "create_time": { "seconds": 1526406431, }, "labels": { "env": "prod", "foo":
  "bar" }, "state": READY, "software_versions": { "software_update": "cloud-
  sql-09-28-2018", }, "maintenance_policy_names": { "UpdatePolicy":
  "projects/123456/locations/us-east1/maintenancePolicies/prod-update-policy",
  } "tenant_project_id": "cloud-sql-test-tenant", "producer_metadata": {
  "cloud-sql-tier": "basic", "cloud-sql-instance-size": "1G", },
  "provisioned_resources": [ { "resource-type": "compute-instance", "resource-
  url": "https://www.googleapis.com/compute/v1/projects/cloud-sql/zones/us-
  east1-b/instances/vm-1", } ], "maintenance_schedules": { "csa_rollout": {
  "start_time": { "seconds": 1526406431, }, "end_time": { "seconds":
  1535406431, }, }, "ncsa_rollout": { "start_time": { "seconds": 1526406431,
  }, "end_time": { "seconds": 1535406431, }, } }, "consumer_defined_name":
  "my-sql-instance1", } ``` LINT.IfChange

  Enums:
    StateValueValuesEnum: Output only. Current lifecycle state of the resource
      (e.g. if it's being created or ready to use).

  Messages:
    LabelsValue: Optional. Resource labels to represent user provided
      metadata. Each label is a key-value pair, where both the key and the
      value are arbitrary strings provided by the user.
    MaintenancePolicyNamesValue: Optional. The MaintenancePolicies that have
      been attached to the instance. The key must be of the type name of the
      oneof policy name defined in MaintenancePolicy, and the referenced
      policy must define the same policy type. For details, please refer to
      go/mr-user-guide. Should not be set if
      maintenance_settings.maintenance_policies is set.
    MaintenanceSchedulesValue: The MaintenanceSchedule contains the scheduling
      information of published maintenance schedule with same key as
      software_versions.
    NotificationParametersValue: Optional. notification_parameter are
      information that service producers may like to include that is not
      relevant to Rollout. This parameter will only be passed to Gamma and
      Cloud Logging for notification/logging purpose.
    ProducerMetadataValue: Output only. Custom string attributes used
      primarily to expose producer-specific information in monitoring
      dashboards. See go/get-instance-metadata.
    SoftwareVersionsValue: Software versions that are used to deploy this
      instance. This can be mutated by rollout services.

  Fields:
    consumerDefinedName: consumer_defined_name is the name of the instance set
      by the service consumers. Generally this is different from the `name`
      field which reperesents the system-assigned id of the instance which the
      service consumers do not recognize. This is a required field for tenants
      onboarding to Maintenance Window notifications (go/slm-rollout-
      maintenance-policies#prerequisites).
    consumerProjectNumber: Optional. The consumer_project_number associated
      with this Apigee instance. This field is added specifically to support
      Apigee integration with SLM Rollout and UMM. It represents the numerical
      project ID of the GCP project that consumes this Apigee instance. It is
      used for SLM rollout notifications and UMM integration, enabling proper
      mapping to customer projects and log delivery for Apigee instances. This
      field complements consumer_project_id and may be used for specific
      Apigee scenarios where the numerical ID is required.
    createTime: Output only. Timestamp when the resource was created.
    instanceType: Optional. The instance_type of this instance of format: proj
      ects/{project_number}/locations/{location_id}/instanceTypes/{instance_ty
      pe_id}. Instance Type represents a high-level tier or SKU of the service
      that this instance belong to. When enabled(eg: Maintenance Rollout),
      Rollout uses 'instance_type' along with 'software_versions' to determine
      whether instance needs an update or not.
    labels: Optional. Resource labels to represent user provided metadata.
      Each label is a key-value pair, where both the key and the value are
      arbitrary strings provided by the user.
    maintenancePolicyNames: Optional. The MaintenancePolicies that have been
      attached to the instance. The key must be of the type name of the oneof
      policy name defined in MaintenancePolicy, and the referenced policy must
      define the same policy type. For details, please refer to go/mr-user-
      guide. Should not be set if maintenance_settings.maintenance_policies is
      set.
    maintenanceSchedules: The MaintenanceSchedule contains the scheduling
      information of published maintenance schedule with same key as
      software_versions.
    maintenanceSettings: Optional. The MaintenanceSettings associated with
      instance.
    name: Unique name of the resource. It uses the form: `projects/{project_nu
      mber}/locations/{location_id}/instances/{instance_id}` Note: This name
      is passed, stored and logged across the rollout system. So use of
      consumer project_id or any other consumer PII in the name is strongly
      discouraged for wipeout (go/wipeout) compliance. See
      go/elysium/project_ids#storage-guidance for more details.
    notificationParameters: Optional. notification_parameter are information
      that service producers may like to include that is not relevant to
      Rollout. This parameter will only be passed to Gamma and Cloud Logging
      for notification/logging purpose.
    producerMetadata: Output only. Custom string attributes used primarily to
      expose producer-specific information in monitoring dashboards. See
      go/get-instance-metadata.
    provisionedResources: Output only. The list of data plane resources
      provisioned for this instance, e.g. compute VMs. See go/get-instance-
      metadata.
    slmInstanceTemplate: Link to the SLM instance template. Only populated
      when updating SLM instances via SSA's Actuation service adaptor. Service
      producers with custom control plane (e.g. Cloud SQL) doesn't need to
      populate this field. Instead they should use software_versions.
    sloMetadata: Output only. SLO metadata for instance classification in the
      Standardized dataplane SLO platform. See go/cloud-ssa-standard-slo for
      feature description.
    softwareVersions: Software versions that are used to deploy this instance.
      This can be mutated by rollout services.
    state: Output only. Current lifecycle state of the resource (e.g. if it's
      being created or ready to use).
    tenantProjectId: Output only. ID of the associated GCP tenant project. See
      go/get-instance-metadata.
    updateTime: Output only. Timestamp when the resource was last modified.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. Current lifecycle state of the resource (e.g. if it's
    being created or ready to use).

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      CREATING: Instance is being created.
      READY: Instance has been created and is ready to use.
      UPDATING: Instance is being updated.
      REPAIRING: Instance is unheathy and under repair.
      DELETING: Instance is being deleted.
      ERROR: Instance encountered an error and is in indeterministic state.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    READY = 2
    UPDATING = 3
    REPAIRING = 4
    DELETING = 5
    ERROR = 6

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Resource labels to represent user provided metadata. Each
    label is a key-value pair, where both the key and the value are arbitrary
    strings provided by the user.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MaintenancePolicyNamesValue(_messages.Message):
    r"""Optional. The MaintenancePolicies that have been attached to the
    instance. The key must be of the type name of the oneof policy name
    defined in MaintenancePolicy, and the referenced policy must define the
    same policy type. For details, please refer to go/mr-user-guide. Should
    not be set if maintenance_settings.maintenance_policies is set.

    Messages:
      AdditionalProperty: An additional property for a
        MaintenancePolicyNamesValue object.

    Fields:
      additionalProperties: Additional properties of type
        MaintenancePolicyNamesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MaintenancePolicyNamesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MaintenanceSchedulesValue(_messages.Message):
    r"""The MaintenanceSchedule contains the scheduling information of
    published maintenance schedule with same key as software_versions.

    Messages:
      AdditionalProperty: An additional property for a
        MaintenanceSchedulesValue object.

    Fields:
      additionalProperties: Additional properties of type
        MaintenanceSchedulesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MaintenanceSchedulesValue object.

      Fields:
        key: Name of the additional property.
        value: A
          GoogleCloudSaasacceleratorManagementProvidersV1MaintenanceSchedule
          attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1MaintenanceSchedule', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class NotificationParametersValue(_messages.Message):
    r"""Optional. notification_parameter are information that service
    producers may like to include that is not relevant to Rollout. This
    parameter will only be passed to Gamma and Cloud Logging for
    notification/logging purpose.

    Messages:
      AdditionalProperty: An additional property for a
        NotificationParametersValue object.

    Fields:
      additionalProperties: Additional properties of type
        NotificationParametersValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a NotificationParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A
          GoogleCloudSaasacceleratorManagementProvidersV1NotificationParameter
          attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1NotificationParameter', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ProducerMetadataValue(_messages.Message):
    r"""Output only. Custom string attributes used primarily to expose
    producer-specific information in monitoring dashboards. See go/get-
    instance-metadata.

    Messages:
      AdditionalProperty: An additional property for a ProducerMetadataValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        ProducerMetadataValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ProducerMetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class SoftwareVersionsValue(_messages.Message):
    r"""Software versions that are used to deploy this instance. This can be
    mutated by rollout services.

    Messages:
      AdditionalProperty: An additional property for a SoftwareVersionsValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        SoftwareVersionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a SoftwareVersionsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  consumerDefinedName = _messages.StringField(1)
  consumerProjectNumber = _messages.StringField(2)
  createTime = _messages.StringField(3)
  instanceType = _messages.StringField(4)
  labels = _messages.MessageField('LabelsValue', 5)
  maintenancePolicyNames = _messages.MessageField('MaintenancePolicyNamesValue', 6)
  maintenanceSchedules = _messages.MessageField('MaintenanceSchedulesValue', 7)
  maintenanceSettings = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1MaintenanceSettings', 8)
  name = _messages.StringField(9)
  notificationParameters = _messages.MessageField('NotificationParametersValue', 10)
  producerMetadata = _messages.MessageField('ProducerMetadataValue', 11)
  provisionedResources = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1ProvisionedResource', 12, repeated=True)
  slmInstanceTemplate = _messages.StringField(13)
  sloMetadata = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1SloMetadata', 14)
  softwareVersions = _messages.MessageField('SoftwareVersionsValue', 15)
  state = _messages.EnumField('StateValueValuesEnum', 16)
  tenantProjectId = _messages.StringField(17)
  updateTime = _messages.StringField(18)


class GoogleCloudSaasacceleratorManagementProvidersV1MaintenanceSchedule(_messages.Message):
  r"""Maintenance schedule which is exposed to customer and potentially end
  user, indicating published upcoming future maintenance schedule

  Fields:
    canReschedule: This field is deprecated, and will be always set to true
      since reschedule can happen multiple times now. This field should not be
      removed until all service producers remove this for their customers.
    endTime: The scheduled end time for the maintenance.
    rolloutManagementPolicy: The rollout management policy this maintenance
      schedule is associated with. When doing reschedule update request, the
      reschedule should be against this given policy.
    scheduleDeadlineTime: schedule_deadline_time is the time deadline any
      schedule start time cannot go beyond, including reschedule. It's
      normally the initial schedule start time plus maintenance window length
      (1 day or 1 week). Maintenance cannot be scheduled to start beyond this
      deadline.
    startTime: The scheduled start time for the maintenance.
  """

  canReschedule = _messages.BooleanField(1)
  endTime = _messages.StringField(2)
  rolloutManagementPolicy = _messages.StringField(3)
  scheduleDeadlineTime = _messages.StringField(4)
  startTime = _messages.StringField(5)


class GoogleCloudSaasacceleratorManagementProvidersV1MaintenanceSettings(_messages.Message):
  r"""Maintenance settings associated with instance. Allows service producers
  and end users to assign settings that controls maintenance on this instance.

  Messages:
    MaintenancePoliciesValue: Optional. The MaintenancePolicies that have been
      attached to the instance. The key must be of the type name of the oneof
      policy name defined in MaintenancePolicy, and the embedded policy must
      define the same policy type. For details, please refer to go/mr-user-
      guide. Should not be set if maintenance_policy_names is set. If only the
      name is needed, then only populate MaintenancePolicy.name.

  Fields:
    exclude: Optional. Exclude instance from maintenance. When true, rollout
      service will not attempt maintenance on the instance. Rollout service
      will include the instance in reported rollout progress as not attempted.
    isRollback: Optional. If the update call is triggered from rollback, set
      the value as true.
    maintenancePolicies: Optional. The MaintenancePolicies that have been
      attached to the instance. The key must be of the type name of the oneof
      policy name defined in MaintenancePolicy, and the embedded policy must
      define the same policy type. For details, please refer to go/mr-user-
      guide. Should not be set if maintenance_policy_names is set. If only the
      name is needed, then only populate MaintenancePolicy.name.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MaintenancePoliciesValue(_messages.Message):
    r"""Optional. The MaintenancePolicies that have been attached to the
    instance. The key must be of the type name of the oneof policy name
    defined in MaintenancePolicy, and the embedded policy must define the same
    policy type. For details, please refer to go/mr-user-guide. Should not be
    set if maintenance_policy_names is set. If only the name is needed, then
    only populate MaintenancePolicy.name.

    Messages:
      AdditionalProperty: An additional property for a
        MaintenancePoliciesValue object.

    Fields:
      additionalProperties: Additional properties of type
        MaintenancePoliciesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MaintenancePoliciesValue object.

      Fields:
        key: Name of the additional property.
        value: A MaintenancePolicy attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('MaintenancePolicy', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  exclude = _messages.BooleanField(1)
  isRollback = _messages.BooleanField(2)
  maintenancePolicies = _messages.MessageField('MaintenancePoliciesValue', 3)


class GoogleCloudSaasacceleratorManagementProvidersV1NodeSloMetadata(_messages.Message):
  r"""Node information for custom per-node SLO implementations. SSA does not
  support per-node SLO, but producers can populate per-node information in
  SloMetadata for custom precomputations. SSA Eligibility Exporter will emit
  per-node metric based on this information.

  Fields:
    location: The location of the node, if different from instance location.
    nodeId: The id of the node. This should be equal to
      SaasInstanceNode.node_id.
    perSliEligibility: If present, this will override eligibility for the node
      coming from instance or exclusions for specified SLIs.
  """

  location = _messages.StringField(1)
  nodeId = _messages.StringField(2)
  perSliEligibility = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1PerSliSloEligibility', 3)


class GoogleCloudSaasacceleratorManagementProvidersV1NotificationParameter(_messages.Message):
  r"""Contains notification related data.

  Fields:
    values: Optional. Array of string values. e.g. instance's replica
      information.
  """

  values = _messages.StringField(1, repeated=True)


class GoogleCloudSaasacceleratorManagementProvidersV1PerSliSloEligibility(_messages.Message):
  r"""PerSliSloEligibility is a mapping from an SLI name to eligibility.

  Messages:
    EligibilitiesValue: An entry in the eligibilities map specifies an
      eligibility for a particular SLI for the given instance. The SLI key in
      the name must be a valid SLI name specified in the Eligibility Exporter
      binary flags otherwise an error will be emitted by Eligibility Exporter
      and the oncaller will be alerted. If an SLI has been defined in the
      binary flags but the eligibilities map does not contain it, the
      corresponding SLI time series will not be emitted by the Eligibility
      Exporter. This ensures a smooth rollout and compatibility between the
      data produced by different versions of the Eligibility Exporters. If
      eligibilities map contains a key for an SLI which has not been declared
      in the binary flags, there will be an error message emitted in the
      Eligibility Exporter log and the metric for the SLI in question will not
      be emitted.

  Fields:
    eligibilities: An entry in the eligibilities map specifies an eligibility
      for a particular SLI for the given instance. The SLI key in the name
      must be a valid SLI name specified in the Eligibility Exporter binary
      flags otherwise an error will be emitted by Eligibility Exporter and the
      oncaller will be alerted. If an SLI has been defined in the binary flags
      but the eligibilities map does not contain it, the corresponding SLI
      time series will not be emitted by the Eligibility Exporter. This
      ensures a smooth rollout and compatibility between the data produced by
      different versions of the Eligibility Exporters. If eligibilities map
      contains a key for an SLI which has not been declared in the binary
      flags, there will be an error message emitted in the Eligibility
      Exporter log and the metric for the SLI in question will not be emitted.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class EligibilitiesValue(_messages.Message):
    r"""An entry in the eligibilities map specifies an eligibility for a
    particular SLI for the given instance. The SLI key in the name must be a
    valid SLI name specified in the Eligibility Exporter binary flags
    otherwise an error will be emitted by Eligibility Exporter and the
    oncaller will be alerted. If an SLI has been defined in the binary flags
    but the eligibilities map does not contain it, the corresponding SLI time
    series will not be emitted by the Eligibility Exporter. This ensures a
    smooth rollout and compatibility between the data produced by different
    versions of the Eligibility Exporters. If eligibilities map contains a key
    for an SLI which has not been declared in the binary flags, there will be
    an error message emitted in the Eligibility Exporter log and the metric
    for the SLI in question will not be emitted.

    Messages:
      AdditionalProperty: An additional property for a EligibilitiesValue
        object.

    Fields:
      additionalProperties: Additional properties of type EligibilitiesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a EligibilitiesValue object.

      Fields:
        key: Name of the additional property.
        value: A GoogleCloudSaasacceleratorManagementProvidersV1SloEligibility
          attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1SloEligibility', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  eligibilities = _messages.MessageField('EligibilitiesValue', 1)


class GoogleCloudSaasacceleratorManagementProvidersV1ProvisionedResource(_messages.Message):
  r"""Describes provisioned dataplane resources.

  Fields:
    resourceType: Type of the resource. This can be either a GCP resource or a
      custom one (e.g. another cloud provider's VM). For GCP compute resources
      use singular form of the names listed in GCP compute API documentation
      (https://cloud.google.com/compute/docs/reference/rest/v1/), prefixed
      with 'compute-', for example: 'compute-instance', 'compute-disk',
      'compute-autoscaler'.
    resourceUrl: URL identifying the resource, e.g.
      "https://www.googleapis.com/compute/v1/projects/...)".
  """

  resourceType = _messages.StringField(1)
  resourceUrl = _messages.StringField(2)


class GoogleCloudSaasacceleratorManagementProvidersV1SloEligibility(_messages.Message):
  r"""SloEligibility is a tuple containing eligibility value: true if an
  instance is eligible for SLO calculation or false if it should be excluded
  from all SLO-related calculations along with a user-defined reason.

  Fields:
    eligible: Whether an instance is eligible or ineligible.
    reason: User-defined reason for the current value of instance eligibility.
      Usually, this can be directly mapped to the internal state. An empty
      reason is allowed.
  """

  eligible = _messages.BooleanField(1)
  reason = _messages.StringField(2)


class GoogleCloudSaasacceleratorManagementProvidersV1SloMetadata(_messages.Message):
  r"""SloMetadata contains resources required for proper SLO classification of
  the instance.

  Fields:
    nodes: Optional. List of nodes. Some producers need to use per-node
      metadata to calculate SLO. This field allows such producers to publish
      per-node SLO meta data, which will be consumed by SSA Eligibility
      Exporter and published in the form of per node metric to Monarch.
    perSliEligibility: Optional. Multiple per-instance SLI eligibilities which
      apply for individual SLIs.
    tier: Name of the SLO tier the Instance belongs to. This name will be
      expected to match the tiers specified in the service SLO configuration.
      Field is mandatory and must not be empty.
  """

  nodes = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1NodeSloMetadata', 1, repeated=True)
  perSliEligibility = _messages.MessageField('GoogleCloudSaasacceleratorManagementProvidersV1PerSliSloEligibility', 2)
  tier = _messages.StringField(3)


class Instance(_messages.Message):
  r"""A Filestore instance.

  Enums:
    StateValueValuesEnum: Output only. The instance state.
    TierValueValuesEnum: The service tier of the instance.

  Messages:
    LabelsValue: Resource labels to represent user provided metadata.

  Fields:
    createTime: Output only. The time when the instance was created.
    description: A description of the instance (2048 characters or less).
    etag: Server-specified ETag for the instance resource to prevent
      simultaneous updates from overwriting each other.
    fileShares: File system shares on the instance. For this version, only a
      single file share is supported.
    labels: Resource labels to represent user provided metadata.
    name: Output only. The resource name of the instance, in the format `proje
      cts/{project_number}/locations/{location_id}/instances/{instance_id}`.
    networks: VPC networks to which the instance is connected. For this
      version, only a single network is supported.
    satisfiesPzi: Output only. Reserved for future use.
    satisfiesPzs: Output only. Reserved for future use.
    state: Output only. The instance state.
    statusMessage: Output only. Additional information about the instance
      state, if available.
    tier: The service tier of the instance.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The instance state.

    Values:
      STATE_UNSPECIFIED: State not set.
      CREATING: The instance is being created.
      READY: The instance is available for use.
      REPAIRING: Work is being done on the instance. You can get further
        details from the `statusMessage` field of the `Instance` resource.
      DELETING: The instance is shutting down.
      ERROR: The instance is experiencing an issue and might be unusable. You
        can get further details from the `statusMessage` field of the
        `Instance` resource.
      RESTORING: The instance is restoring a snapshot or backup to an existing
        file share and may be unusable during this time.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    READY = 2
    REPAIRING = 3
    DELETING = 4
    ERROR = 5
    RESTORING = 6

  class TierValueValuesEnum(_messages.Enum):
    r"""The service tier of the instance.

    Values:
      TIER_UNSPECIFIED: Not set.
      STANDARD: STANDARD tier. BASIC_HDD is the preferred term for this tier.
      PREMIUM: PREMIUM tier. BASIC_SSD is the preferred term for this tier.
      BASIC_HDD: BASIC instances offer a maximum capacity of 63.9 TB.
        BASIC_HDD is an alias for STANDARD Tier, offering economical
        performance backed by HDD.
      BASIC_SSD: BASIC instances offer a maximum capacity of 63.9 TB.
        BASIC_SSD is an alias for PREMIUM Tier, and offers improved
        performance backed by SSD.
      HIGH_SCALE_SSD: HIGH_SCALE instances offer expanded capacity and
        performance scaling capabilities.
      ENTERPRISE: ENTERPRISE instances offer the features and availability
        needed for mission-critical workloads.
      ZONAL: ZONAL instances offer large/small capacities and performance
        scaling capabilities.
      REGIONAL: REGIONAL instances offer the features and availability needed
        for mission-critical workloads.
    """
    TIER_UNSPECIFIED = 0
    STANDARD = 1
    PREMIUM = 2
    BASIC_HDD = 3
    BASIC_SSD = 4
    HIGH_SCALE_SSD = 5
    ENTERPRISE = 6
    ZONAL = 7
    REGIONAL = 8

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Resource labels to represent user provided metadata.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  description = _messages.StringField(2)
  etag = _messages.StringField(3)
  fileShares = _messages.MessageField('FileShareConfig', 4, repeated=True)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  networks = _messages.MessageField('NetworkConfig', 7, repeated=True)
  satisfiesPzi = _messages.BooleanField(8)
  satisfiesPzs = _messages.BooleanField(9)
  state = _messages.EnumField('StateValueValuesEnum', 10)
  statusMessage = _messages.StringField(11)
  tier = _messages.EnumField('TierValueValuesEnum', 12)


class ListBackupsResponse(_messages.Message):
  r"""ListBackupsResponse is the result of ListBackupsRequest.

  Fields:
    backups: A list of backups in the project for the specified location. If
      the `{location}` value in the request is "-", the response contains a
      list of backups from all locations. If any location is unreachable, the
      response will only return backups in reachable locations and the
      "unreachable" field will be populated with a list of unreachable
      locations.
    nextPageToken: The token you can use to retrieve the next page of results.
      Not returned if there are no more results in the list.
    unreachable: Locations that could not be reached.
  """

  backups = _messages.MessageField('Backup', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListInstancesResponse(_messages.Message):
  r"""ListInstancesResponse is the result of ListInstancesRequest.

  Fields:
    instances: A list of instances in the project for the specified location.
      If the `{location}` value in the request is "-", the response contains a
      list of instances from all locations. If any location is unreachable,
      the response will only return instances in reachable locations and the
      "unreachable" field will be populated with a list of unreachable
      locations.
    nextPageToken: The token you can use to retrieve the next page of results.
      Not returned if there are no more results in the list.
    unreachable: Locations that could not be reached.
  """

  instances = _messages.MessageField('Instance', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListSnapshotsResponse(_messages.Message):
  r"""ListSnapshotsResponse is the result of ListSnapshotsRequest.

  Fields:
    nextPageToken: The token you can use to retrieve the next page of results.
      Not returned if there are no more results in the list.
    snapshots: A list of snapshots in the project for the specified location.
      If the `{location}` value in the request is "-", the response contains a
      list of snapshots from all locations. If any location is unreachable,
      the response will only return snapshots in reachable locations and the
      "unreachable_locations" field will be populated with a list of
      unreachable locations.
    unreachableLocations: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  snapshots = _messages.MessageField('Snapshot', 2, repeated=True)
  unreachableLocations = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class MaintenancePolicy(_messages.Message):
  r"""Defines policies to service maintenance events.

  Enums:
    StateValueValuesEnum: Optional. The state of the policy.

  Messages:
    LabelsValue: Optional. Resource labels to represent user provided
      metadata. Each label is a key-value pair, where both the key and the
      value are arbitrary strings provided by the user.

  Fields:
    createTime: Output only. The time when the resource was created.
    description: Optional. Description of what this policy is for.
      Create/Update methods return INVALID_ARGUMENT if the length is greater
      than 512.
    labels: Optional. Resource labels to represent user provided metadata.
      Each label is a key-value pair, where both the key and the value are
      arbitrary strings provided by the user.
    name: Required. MaintenancePolicy name using the form: `projects/{project_
      id}/locations/{location_id}/maintenancePolicies/{maintenance_policy_id}`
      where {project_id} refers to a GCP consumer project ID, {location_id}
      refers to a GCP region/zone, {maintenance_policy_id} must be 1-63
      characters long and match the regular expression
      `[a-z0-9]([-a-z0-9]*[a-z0-9])?`.
    state: Optional. The state of the policy.
    updatePolicy: Maintenance policy applicable to instance update.
    updateTime: Output only. The time when the resource was updated.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Optional. The state of the policy.

    Values:
      STATE_UNSPECIFIED: Unspecified state.
      READY: Resource is ready to be used.
      DELETING: Resource is being deleted. It can no longer be attached to
        instances.
    """
    STATE_UNSPECIFIED = 0
    READY = 1
    DELETING = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Resource labels to represent user provided metadata. Each
    label is a key-value pair, where both the key and the value are arbitrary
    strings provided by the user.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  description = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  name = _messages.StringField(4)
  state = _messages.EnumField('StateValueValuesEnum', 5)
  updatePolicy = _messages.MessageField('UpdatePolicy', 6)
  updateTime = _messages.StringField(7)


class MaintenanceWindow(_messages.Message):
  r"""MaintenanceWindow definition.

  Fields:
    dailyCycle: Daily cycle.
    weeklyCycle: Weekly cycle.
  """

  dailyCycle = _messages.MessageField('DailyCycle', 1)
  weeklyCycle = _messages.MessageField('WeeklyCycle', 2)


class NetworkConfig(_messages.Message):
  r"""Network configuration for the instance.

  Enums:
    ConnectModeValueValuesEnum: The network connect mode of the Filestore
      instance. If not provided, the connect mode defaults to DIRECT_PEERING.
    ModesValueListEntryValuesEnum:

  Fields:
    connectMode: The network connect mode of the Filestore instance. If not
      provided, the connect mode defaults to DIRECT_PEERING.
    ipAddresses: Output only. IPv4 addresses in the format
      `{octet1}.{octet2}.{octet3}.{octet4}` or IPv6 addresses in the format `{
      block1}:{block2}:{block3}:{block4}:{block5}:{block6}:{block7}:{block8}`.
    modes: Internet protocol versions for which the instance has IP addresses
      assigned. For this version, only MODE_IPV4 is supported.
    network: The name of the Google Compute Engine [VPC
      network](https://cloud.google.com/vpc/docs/vpc) to which the instance is
      connected.
    reservedIpRange: Optional, reserved_ip_range can have one of the following
      two types of values. * CIDR range value when using DIRECT_PEERING
      connect mode. * [Allocated IP address
      range](https://cloud.google.com/compute/docs/ip-addresses/reserve-
      static-internal-ip-address) when using PRIVATE_SERVICE_ACCESS connect
      mode. When the name of an allocated IP address range is specified, it
      must be one of the ranges associated with the private service access
      connection. When specified as a direct CIDR value, it must be a /29 CIDR
      block for Basic tier, a /24 CIDR block for High Scale tier, or a /26
      CIDR block for Enterprise tier in one of the [internal IP address ranges
      ](https://www.arin.net/reference/research/statistics/address_filters/)
      that identifies the range of IP addresses reserved for this instance.
      For example, 10.0.0.0/29, 192.168.0.0/24 or 192.168.0.0/26,
      respectively. The range you specify can't overlap with either existing
      subnets or assigned IP address ranges for other Filestore instances in
      the selected VPC network.
  """

  class ConnectModeValueValuesEnum(_messages.Enum):
    r"""The network connect mode of the Filestore instance. If not provided,
    the connect mode defaults to DIRECT_PEERING.

    Values:
      CONNECT_MODE_UNSPECIFIED: ConnectMode not set.
      DIRECT_PEERING: Connect via direct peering to the Filestore service.
      PRIVATE_SERVICE_ACCESS: Connect to your Filestore instance using Private
        Service Access. Private services access provides an IP address range
        for multiple Google Cloud services, including Filestore.
    """
    CONNECT_MODE_UNSPECIFIED = 0
    DIRECT_PEERING = 1
    PRIVATE_SERVICE_ACCESS = 2

  class ModesValueListEntryValuesEnum(_messages.Enum):
    r"""ModesValueListEntryValuesEnum enum type.

    Values:
      ADDRESS_MODE_UNSPECIFIED: Internet protocol not set.
      MODE_IPV4: Use the IPv4 internet protocol.
    """
    ADDRESS_MODE_UNSPECIFIED = 0
    MODE_IPV4 = 1

  connectMode = _messages.EnumField('ConnectModeValueValuesEnum', 1)
  ipAddresses = _messages.StringField(2, repeated=True)
  modes = _messages.EnumField('ModesValueListEntryValuesEnum', 3, repeated=True)
  network = _messages.StringField(4)
  reservedIpRange = _messages.StringField(5)


class NfsExportOptions(_messages.Message):
  r"""NFS export options specifications.

  Enums:
    AccessModeValueValuesEnum: Either READ_ONLY, for allowing only read
      requests on the exported directory, or READ_WRITE, for allowing both
      read and write requests. The default is READ_WRITE.
    SquashModeValueValuesEnum: Either NO_ROOT_SQUASH, for allowing root access
      on the exported directory, or ROOT_SQUASH, for not allowing root access.
      The default is NO_ROOT_SQUASH.

  Fields:
    accessMode: Either READ_ONLY, for allowing only read requests on the
      exported directory, or READ_WRITE, for allowing both read and write
      requests. The default is READ_WRITE.
    anonGid: An integer representing the anonymous group id with a default
      value of 65534. Anon_gid may only be set with squash_mode of
      ROOT_SQUASH. An error will be returned if this field is specified for
      other squash_mode settings.
    anonUid: An integer representing the anonymous user id with a default
      value of 65534. Anon_uid may only be set with squash_mode of
      ROOT_SQUASH. An error will be returned if this field is specified for
      other squash_mode settings.
    ipRanges: List of either an IPv4 addresses in the format
      `{octet1}.{octet2}.{octet3}.{octet4}` or CIDR ranges in the format
      `{octet1}.{octet2}.{octet3}.{octet4}/{mask size}` which may mount the
      file share. Overlapping IP ranges are not allowed, both within and
      across NfsExportOptions. An error will be returned. The limit is 128 IP
      ranges/addresses for each FileShareConfig among all NfsExportOptions.
    squashMode: Either NO_ROOT_SQUASH, for allowing root access on the
      exported directory, or ROOT_SQUASH, for not allowing root access. The
      default is NO_ROOT_SQUASH.
  """

  class AccessModeValueValuesEnum(_messages.Enum):
    r"""Either READ_ONLY, for allowing only read requests on the exported
    directory, or READ_WRITE, for allowing both read and write requests. The
    default is READ_WRITE.

    Values:
      ACCESS_MODE_UNSPECIFIED: AccessMode not set.
      READ_ONLY: The client can only read the file share.
      READ_WRITE: The client can read and write the file share (default).
    """
    ACCESS_MODE_UNSPECIFIED = 0
    READ_ONLY = 1
    READ_WRITE = 2

  class SquashModeValueValuesEnum(_messages.Enum):
    r"""Either NO_ROOT_SQUASH, for allowing root access on the exported
    directory, or ROOT_SQUASH, for not allowing root access. The default is
    NO_ROOT_SQUASH.

    Values:
      SQUASH_MODE_UNSPECIFIED: SquashMode not set.
      NO_ROOT_SQUASH: The Root user has root access to the file share
        (default).
      ROOT_SQUASH: The Root user has squashed access to the anonymous uid/gid.
    """
    SQUASH_MODE_UNSPECIFIED = 0
    NO_ROOT_SQUASH = 1
    ROOT_SQUASH = 2

  accessMode = _messages.EnumField('AccessModeValueValuesEnum', 1)
  anonGid = _messages.IntegerField(2)
  anonUid = _messages.IntegerField(3)
  ipRanges = _messages.StringField(4, repeated=True)
  squashMode = _messages.EnumField('SquashModeValueValuesEnum', 5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    cancelRequested: Output only. Identifies whether the user has requested
      cancellation of the operation. Operations that have been cancelled
      successfully have google.longrunning.Operation.error value with a
      google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    statusDetail: Output only. Human-readable status of the operation, if any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  cancelRequested = _messages.BooleanField(2)
  createTime = _messages.StringField(3)
  endTime = _messages.StringField(4)
  statusDetail = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class RestoreInstanceRequest(_messages.Message):
  r"""RestoreInstanceRequest restores an existing instance's file share from a
  snapshot or backup.

  Fields:
    fileShare: Required. Name of the file share in the Filestore instance that
      the snapshot is being restored to.
    sourceBackup: The resource name of the backup, in the format
      `projects/{project_id}/locations/{location_id}/backups/{backup_id}`.
    sourceSnapshot: The resource name of the snapshot, in the format
      `projects/{project_id}/locations/{location_id}/snapshots/{snapshot_id}`.
  """

  fileShare = _messages.StringField(1)
  sourceBackup = _messages.StringField(2)
  sourceSnapshot = _messages.StringField(3)


class Schedule(_messages.Message):
  r"""Configure the schedule.

  Enums:
    DayValueValuesEnum: Allows to define schedule that runs specified day of
      the week.

  Fields:
    day: Allows to define schedule that runs specified day of the week.
    duration: Output only. Duration of the time window, set by service
      producer.
    startTime: Time within the window to start the operations.
  """

  class DayValueValuesEnum(_messages.Enum):
    r"""Allows to define schedule that runs specified day of the week.

    Values:
      DAY_OF_WEEK_UNSPECIFIED: The day of the week is unspecified.
      MONDAY: Monday
      TUESDAY: Tuesday
      WEDNESDAY: Wednesday
      THURSDAY: Thursday
      FRIDAY: Friday
      SATURDAY: Saturday
      SUNDAY: Sunday
    """
    DAY_OF_WEEK_UNSPECIFIED = 0
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7

  day = _messages.EnumField('DayValueValuesEnum', 1)
  duration = _messages.StringField(2)
  startTime = _messages.MessageField('TimeOfDay', 3)


class Snapshot(_messages.Message):
  r"""A Filestore snapshot.

  Enums:
    SourceInstanceTierValueValuesEnum: Output only. The service tier of the
      source Filestore instance that this snapshot is created from.
    StateValueValuesEnum: Output only. The snapshot state.
    TypeValueValuesEnum: The Type of the Snapshot.

  Messages:
    LabelsValue: Resource labels to represent user provided metadata.

  Fields:
    capacityGb: Output only. Capacity of the source file share when the
      snapshot was created.
    createTime: Output only. The time when the snapshot was created.
    description: A description of the snapshot with 2048 characters or less.
      Requests with longer descriptions will be rejected.
    labels: Resource labels to represent user provided metadata.
    name: Output only. The resource name of the snapshot, in the format
      `projects/{project_id}/locations/{location_id}/snapshots/{snapshot_id}`.
    sourceFileShare: Name of the file share in the source Filestore instance
      that the snapshot is created from.
    sourceInstance: The resource name of the source Filestore instance, in the
      format
      `projects/{project_id}/locations/{location_id}/instances/{instance_id}`,
      used to create this snapshot.
    sourceInstanceTier: Output only. The service tier of the source Filestore
      instance that this snapshot is created from.
    state: Output only. The snapshot state.
    storageBytes: Output only. The size of the storage used by the snapshot.
      As snapshots share storage, this number is expected to change with
      snapshot creation/deletion. Always equals to capacityGb for local
      snapshots.
    type: The Type of the Snapshot.
  """

  class SourceInstanceTierValueValuesEnum(_messages.Enum):
    r"""Output only. The service tier of the source Filestore instance that
    this snapshot is created from.

    Values:
      TIER_UNSPECIFIED: Not set.
      STANDARD: STANDARD tier. BASIC_HDD is the preferred term for this tier.
      PREMIUM: PREMIUM tier. BASIC_SSD is the preferred term for this tier.
      BASIC_HDD: BASIC instances offer a maximum capacity of 63.9 TB.
        BASIC_HDD is an alias for STANDARD Tier, offering economical
        performance backed by HDD.
      BASIC_SSD: BASIC instances offer a maximum capacity of 63.9 TB.
        BASIC_SSD is an alias for PREMIUM Tier, and offers improved
        performance backed by SSD.
      HIGH_SCALE_SSD: HIGH_SCALE instances offer expanded capacity and
        performance scaling capabilities.
      ENTERPRISE: ENTERPRISE instances offer the features and availability
        needed for mission-critical workloads.
      ZONAL: ZONAL instances offer large/small capacities and performance
        scaling capabilities.
      REGIONAL: REGIONAL instances offer the features and availability needed
        for mission-critical workloads.
    """
    TIER_UNSPECIFIED = 0
    STANDARD = 1
    PREMIUM = 2
    BASIC_HDD = 3
    BASIC_SSD = 4
    HIGH_SCALE_SSD = 5
    ENTERPRISE = 6
    ZONAL = 7
    REGIONAL = 8

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The snapshot state.

    Values:
      STATE_UNSPECIFIED: State not set.
      CREATING: Point-in-time snapshot is being created.
      FINALIZING: Point-in-time snapshot has been taken and the operation is
        being finalized. At this point, changes to the file share will not be
        reflected in the snapshot.
      READY: Snapshot is available for use.
      DELETING: Snapshot is being deleted.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    FINALIZING = 2
    READY = 3
    DELETING = 4

  class TypeValueValuesEnum(_messages.Enum):
    r"""The Type of the Snapshot.

    Values:
      TYPE_UNSPECIFIED: Not set, will use the default value of BACKUP.
      BACKUP: Backup snapshots are standalone and replicated regionally. Basic
        Instances can be created or restored from this snapshot variant
        (Backup snapshots are not supported for Advanced instances at this
        time).
      LOCAL: Local snapshots are standalone, zonal resources. Basic Instances
        can be created or restored from this snapshot variant (Local snapshots
        are not supported for Advanced instances at this time).
      IN_PLACE: In place snapshots are part of the Instance storage system and
        occupy Instance storage space. They are also tied to the Instance
        lifecycle and are deleted at Instance deletion time. Instances cannot
        be restored or created from this snapshot variant. In Place snapshots
        are only supported for Advanced Instances at this time.
    """
    TYPE_UNSPECIFIED = 0
    BACKUP = 1
    LOCAL = 2
    IN_PLACE = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Resource labels to represent user provided metadata.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  capacityGb = _messages.IntegerField(1)
  createTime = _messages.StringField(2)
  description = _messages.StringField(3)
  labels = _messages.MessageField('LabelsValue', 4)
  name = _messages.StringField(5)
  sourceFileShare = _messages.StringField(6)
  sourceInstance = _messages.StringField(7)
  sourceInstanceTier = _messages.EnumField('SourceInstanceTierValueValuesEnum', 8)
  state = _messages.EnumField('StateValueValuesEnum', 9)
  storageBytes = _messages.IntegerField(10)
  type = _messages.EnumField('TypeValueValuesEnum', 11)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TimeOfDay(_messages.Message):
  r"""Represents a time of day. The date and time zone are either not
  significant or are specified elsewhere. An API may choose to allow leap
  seconds. Related types are google.type.Date and `google.protobuf.Timestamp`.

  Fields:
    hours: Hours of a day in 24 hour format. Must be greater than or equal to
      0 and typically must be less than or equal to 23. An API may choose to
      allow the value "24:00:00" for scenarios like business closing time.
    minutes: Minutes of an hour. Must be greater than or equal to 0 and less
      than or equal to 59.
    nanos: Fractions of seconds, in nanoseconds. Must be greater than or equal
      to 0 and less than or equal to 999,999,999.
    seconds: Seconds of a minute. Must be greater than or equal to 0 and
      typically must be less than or equal to 59. An API may allow the value
      60 if it allows leap-seconds.
  """

  hours = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  minutes = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  nanos = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  seconds = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class UpdatePolicy(_messages.Message):
  r"""Maintenance policy applicable to instance updates.

  Enums:
    ChannelValueValuesEnum: Optional. Relative scheduling channel applied to
      resource.

  Fields:
    channel: Optional. Relative scheduling channel applied to resource.
    denyMaintenancePeriods: Deny Maintenance Period that is applied to
      resource to indicate when maintenance is forbidden. The protocol
      supports zero-to-many such periods, but the current SLM Rollout
      implementation only supports zero-to-one.
    window: Optional. Maintenance window that is applied to resources covered
      by this policy.
  """

  class ChannelValueValuesEnum(_messages.Enum):
    r"""Optional. Relative scheduling channel applied to resource.

    Values:
      UPDATE_CHANNEL_UNSPECIFIED: Unspecified channel.
      EARLIER: Early channel within a customer project.
      LATER: Later channel within a customer project.
      WEEK1: ! ! The follow channels can ONLY be used if you adopt the new MW
        system! ! ! NOTE: all WEEK channels are assumed to be under a weekly
        window. ! There is currently no dedicated channel definitions for
        Daily windows. ! If you use Daily window, the system will assume a 1d
        (24Hours) advanced ! notification period b/w EARLY and LATER. ! We may
        consider support more flexible daily channel specifications in ! the
        future. WEEK1 == EARLIER with minimum 7d advanced notification. {7d,
        14d} The system will treat them equally and will use WEEK1 whenever it
        can. New customers are encouraged to use this channel annotation.
      WEEK2: WEEK2 == LATER with minimum 14d advanced notification {14d, 21d}.
      WEEK5: WEEK5 == 40d support. minimum 35d advanced notification {35d,
        42d}.
    """
    UPDATE_CHANNEL_UNSPECIFIED = 0
    EARLIER = 1
    LATER = 2
    WEEK1 = 3
    WEEK2 = 4
    WEEK5 = 5

  channel = _messages.EnumField('ChannelValueValuesEnum', 1)
  denyMaintenancePeriods = _messages.MessageField('DenyMaintenancePeriod', 2, repeated=True)
  window = _messages.MessageField('MaintenanceWindow', 3)


class WeeklyCycle(_messages.Message):
  r"""Time window specified for weekly operations.

  Fields:
    schedule: User can specify multiple windows in a week. Minimum of 1
      window.
  """

  schedule = _messages.MessageField('Schedule', 1, repeated=True)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



"""Generated client library for cloudapiregistry version v1beta."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.cloudapiregistry.v1beta import cloudapiregistry_v1beta_messages as messages


class CloudapiregistryV1beta(base_api.BaseApiClient):
  """Generated client library for service cloudapiregistry version v1beta."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://cloudapiregistry.googleapis.com/'
  MTLS_BASE_URL = 'https://cloudapiregistry.mtls.googleapis.com/'

  _PACKAGE = 'cloudapiregistry'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'CloudapiregistryV1beta'
  _URL_VERSION = 'v1beta'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new cloudapiregistry handle."""
    url = url or self.BASE_URL
    super(CloudapiregistryV1beta, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_mcpServers_mcpTools = self.ProjectsLocationsMcpServersMcpToolsService(self)
    self.projects_locations_mcpServers = self.ProjectsLocationsMcpServersService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsMcpServersMcpToolsService(base_api.BaseApiService):
    """Service class for the projects_locations_mcpServers_mcpTools resource."""

    _NAME = 'projects_locations_mcpServers_mcpTools'

    def __init__(self, client):
      super(CloudapiregistryV1beta.ProjectsLocationsMcpServersMcpToolsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Deprecated: The GetMcpTool method is deprecated and will be removed in a future release. For similar functionality, you can use Agent Registry. Note: Agent Registry does not treat tools as a separate collection. To view the tools exposed by an MCP server, use the ListMcpServers or GetMcpServer methods in the Agent Registry API. For more information, see: https://docs.cloud.google.com/agent-registry/reference/rpc Gets a single McpTool.

      Args:
        request: (CloudapiregistryProjectsLocationsMcpServersMcpToolsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (McpTool) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/mcpServers/{mcpServersId}/mcpTools/{mcpToolsId}',
        http_method='GET',
        method_id='cloudapiregistry.projects.locations.mcpServers.mcpTools.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudapiregistryProjectsLocationsMcpServersMcpToolsGetRequest',
        response_type_name='McpTool',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Deprecated: The ListMcpTools method is deprecated and will be removed in a future release. For similar functionality, you can use Agent Registry. Note: AgentRegistry only lists MCP servers for service APIs that are enabled in a project. For more information, see: https://docs.cloud.google.com/agent-registry/reference/rpc Lists McpTools in a given McpServer.

      Args:
        request: (CloudapiregistryProjectsLocationsMcpServersMcpToolsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListMcpToolsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/mcpServers/{mcpServersId}/mcpTools',
        http_method='GET',
        method_id='cloudapiregistry.projects.locations.mcpServers.mcpTools.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/mcpTools',
        request_field='',
        request_type_name='CloudapiregistryProjectsLocationsMcpServersMcpToolsListRequest',
        response_type_name='ListMcpToolsResponse',
        supports_download=False,
    )

  class ProjectsLocationsMcpServersService(base_api.BaseApiService):
    """Service class for the projects_locations_mcpServers resource."""

    _NAME = 'projects_locations_mcpServers'

    def __init__(self, client):
      super(CloudapiregistryV1beta.ProjectsLocationsMcpServersService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Deprecated: The GetMcpServer method is deprecated and will be removed in a future release. Use AgentRegistry.GetMcpServer instead. For more information, see: https://docs.cloud.google.com/agent-registry/reference/rpc Gets a single McpServer.

      Args:
        request: (CloudapiregistryProjectsLocationsMcpServersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (McpServer) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/mcpServers/{mcpServersId}',
        http_method='GET',
        method_id='cloudapiregistry.projects.locations.mcpServers.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudapiregistryProjectsLocationsMcpServersGetRequest',
        response_type_name='McpServer',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Deprecated: The ListMcpServers method is deprecated and will be removed in a future release. Use AgentRegistry. ListMcpServers instead. Note: AgentRegistry only lists MCP servers for service APIs that are enabled in a project. For more information, see: https://docs.cloud.google.com/agent-registry/reference/rpc Lists McpServers in a given Project.

      Args:
        request: (CloudapiregistryProjectsLocationsMcpServersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListMcpServersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/mcpServers',
        http_method='GET',
        method_id='cloudapiregistry.projects.locations.mcpServers.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/mcpServers',
        request_field='',
        request_type_name='CloudapiregistryProjectsLocationsMcpServersListRequest',
        response_type_name='ListMcpServersResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(CloudapiregistryV1beta.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (CloudapiregistryProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='cloudapiregistry.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='CloudapiregistryProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (CloudapiregistryProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations',
        http_method='GET',
        method_id='cloudapiregistry.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+name}/locations',
        request_field='',
        request_type_name='CloudapiregistryProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(CloudapiregistryV1beta.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to validate orchestration pipeline configurations."""

import pathlib
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.orchestration_pipelines import deployment_model
from googlecloudsdk.command_lib.orchestration_pipelines import gcp_deployer
from googlecloudsdk.command_lib.orchestration_pipelines.handlers import registry
from googlecloudsdk.command_lib.orchestration_pipelines.tools import gcs_utils
from googlecloudsdk.command_lib.orchestration_pipelines.tools import yaml_processor


def _validate_artifact_bucket(env, _, pipeline_models):
  """Validates that the GCS bucket for the environment exists and is accessible."""
  if not (
      env
      and env.artifact_storage
      and env.artifact_storage.bucket
      and pipeline_models
  ):
    return

  gcs_utils.ValidateBucketExistsAndHasPermissions(env.artifact_storage.bucket)


DEPLOYMENT_FILE_NAME = "deployment.yaml"


@calliope_base.Hidden
@calliope_base.DefaultUniverseOnly
@calliope_base.ReleaseTracks(calliope_base.ReleaseTrack.BETA)
class Validate(calliope_base.Command):
  """Validate orchestration pipeline configurations against schemas."""

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        "--environment",
        help="The target environment of the pipeline, as defined in"
        " deployment.yaml.",
    )
    parser.add_argument(
        "--pipeline-paths",
        metavar="PATH",
        type=arg_parsers.ArgList(),
        help="The list of relative pipeline YAML file paths to validate.",
    )

    parser.add_argument(
        "--mode",
        choices=["syntax-only", "full"],
        default="full",
        help=(
            "The validation mode to use. 'syntax-only' checks the syntax"
            " and type correctness of the pipeline YAML files. 'full' performs"
            " 'syntax-only' validation and adds semantic checks for referenced"
            " resources such as Cloud project, Composer environment, and"
            " other resources in both deployment and pipeline YAML files."
            " Default mode is 'full'."
        ),
    )
    yaml_processor.add_substitution_flags(parser)

  def Run(self, args):
    work_dir = pathlib.Path.cwd()

    # 1. Collect all variables from substitutions file and environment variables
    external_vars = yaml_processor.collect_external_vars(args, work_dir)

    # 2. Setup validation contexts based on provided arguments.
    # Each context contains an environment, combined variables, pipeline paths,
    # and resources specific to that context.
    validation_contexts = []
    error_environments = {}
    if args.environment:
      deployment_path = work_dir / DEPLOYMENT_FILE_NAME
      _, combined_variables, environment = yaml_processor.parse_deployment(
          str(deployment_path), args.environment, external_vars
      )
      pipeline_paths_in_deployment = (
          [p.source for p in environment.pipelines]
          if getattr(environment, "pipelines", None)
          else []
      )

      # Check if all the pipeline paths specified in the command are a
      # subset of the pipelines in the deployment environment. If not, raise
      # an error.
      if not args.pipeline_paths:
        pipeline_paths = pipeline_paths_in_deployment
      else:
        pipeline_paths = args.pipeline_paths
        for pipeline_path in pipeline_paths:
          if pipeline_path not in pipeline_paths_in_deployment:
            raise calliope_exceptions.BadArgumentException(
                "--pipeline-paths",
                f"Pipeline path '{pipeline_path}' not found in deployment "
                f"environment '{args.environment}'."
            )
      validation_contexts.append({
          "env": environment,
          "combined_vars": combined_variables,
          "pipeline_paths": pipeline_paths,
          "resources": environment.resources if environment else [],
          "name": args.environment,
      })
    elif args.pipeline_paths:
      # Validating pipelines without an explicit environment context.
      validation_contexts.append({
          "env": None,
          "combined_vars": external_vars,
          "pipeline_paths": args.pipeline_paths,
          "resources": [],
          "name": "provided pipelines",
      })
    else:
      all_environments_dict, error_environments = (
          yaml_processor.load_all_environments(
              work_dir / DEPLOYMENT_FILE_NAME,
              external_vars,
          )
      )
      if not all_environments_dict:
        print("No environments found in deployment.yaml.")
        return

      for env_name, env in all_environments_dict.items():
        yaml_processor.validate_environment(env, env_name)
        combined_variables = {
            "project": env.project,
            "region": env.region,
            **(env.variables if env.variables else {}),
        }
        pipeline_paths = (
            [p.source for p in env.pipelines]
            if getattr(env, "pipelines", None)
            else []
        )
        validation_contexts.append({
            "env": env,
            "combined_vars": combined_variables,
            "pipeline_paths": pipeline_paths,
            "resources": env.resources if env else [],
            "name": env_name,
        })

    if not validation_contexts:
      print("No pipelines or environments to validate.")
      return

    # 3. Perform L1 syntax validation for each context.
    for context in validation_contexts:
      env = context["env"]
      combined_vars = context["combined_vars"]
      pipeline_paths = context["pipeline_paths"]
      resources = context["resources"]
      env_name = context["name"]
      has_environment = env is not None
      pipeline_models = []

      captured_vars = set()
      if env:
        for r in env.resources:
          if isinstance(r, deployment_model.ResourceModel):
            for c in r.capture:
              captured_vars.add(c.variable)

      if pipeline_paths:
        pipeline_models = yaml_processor.validate_pipeline_l1(
            work_dir,
            pipeline_paths,
            combined_vars,
            allowed_missing=captured_vars,
        )
      context["pipeline_models"] = pipeline_models

      if has_environment:
        for resource in resources:
          if resource.type == "resourceProfile":
            continue
          handler = registry.GetHandler(resource, env)
          gcp_deployer.validate_gcp_resource_l1(handler)

      if has_environment:
        print(
            "Successfully finished syntax validation for pipelines and"
            f" resources in deployment environment '{env_name}'."
        )
      elif args.pipeline_paths:
        print(
            "Successfully finished syntax validation for all provided"
            " pipelines."
        )

    if error_environments:
      print("Errors found while parsing deployment file:")
      for env, e in error_environments.items():
        print(f"  Environment '{env}': {e}")

    if args.mode == "syntax-only":
      return

    # 4. Perform L2 semantic validation for each context.
    for context in validation_contexts:
      env = context["env"]
      pipeline_models = context["pipeline_models"]
      resources = context["resources"]
      env_name = context["name"]
      combined_vars = context["combined_vars"]
      has_environment = env is not None

      _validate_artifact_bucket(env, env_name, pipeline_models)

      if pipeline_models:
        yaml_processor.validate_pipeline_l2(
            work_dir,
            pipeline_models,
            env,
        )

      if has_environment:
        for resource in resources:
          if resource.type == "resourceProfile":
            continue
          handler = registry.GetHandler(resource, env)
          gcp_deployer.validate_gcp_resource_l2(handler, combined_vars)

      if has_environment:
        print(
            "Successfully finished full validation for all pipelines and"
            f" resources in deployment environment '{env_name}'."
        )
      elif args.pipeline_paths:
        print(
            "Successfully finished full validation for all provided pipelines."
        )



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Set DAG timeout on workflow template command."""


from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.command_lib.dataproc import workflow_templates
from googlecloudsdk.core import log

DETAILED_HELP = {
    'EXAMPLES':
        """\
      To add a DAG timeout of 2h (or update an existing one to 2h) on a workflow
      template named ``my-workflow-template'' in region ``us-central1'', run:

        $ {command} my-workflow-template --region=us-central1 --dag-timeout=2h"
      """,
}


class SetDagTimeout(base.CreateCommand):
  """Set DAG timeout on a workflow template."""

  detailed_help = DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    dataproc = dp.Dataproc(cls.ReleaseTrack())
    workflow_templates.AddDagTimeoutFlag(parser, True)
    flags.AddTemplateResourceArg(parser, 'set the DAG timeout on',
                                 dataproc.api_version)

  def Run(self, args):
    dataproc = dp.Dataproc(self.ReleaseTrack())

    template_ref = args.CONCEPTS.template.Parse()

    workflow_template = dataproc.GetRegionsWorkflowTemplate(
        template_ref, args.version)

    workflow_template.dagTimeout = str(args.dag_timeout) + 's'

    response = dataproc.client.projects_regions_workflowTemplates.Update(
        workflow_template)

    log.status.Print('Set a DAG timeout of {0} on {1}.'.format(
        workflow_template.dagTimeout, template_ref.Name()))
    return response



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API wrapper for `gcloud network-security firewall-endpoints` commands."""

import datetime

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources


_API_VERSION_FOR_TRACK = {
    base.ReleaseTrack.ALPHA: 'v1alpha1',
    base.ReleaseTrack.BETA: 'v1beta1',
}
ORG_OPERATIONS_COLLECTION = 'networksecurity.organizations.locations.operations'


class Client(object):
  """API client for firewall endpoints commands."""

  def __init__(self, release_track=base.ReleaseTrack.ALPHA):
    self.api_version = _API_VERSION_FOR_TRACK.get(release_track)
    self._client = apis.GetClientInstance('networksecurity', self.api_version)
    self.messages = self._client.MESSAGES_MODULE
    self._service = (
        self._client.organizations_locations_firewallEndpoints_wildfireVerdictChangeRequests
    )
    self._operations_client = self._client.organizations_locations_operations
    self._resource_parser = resources.Registry()
    self._resource_parser.RegisterApiByName('networksecurity', self.api_version)

  def CreateVerdictChangeRequest(self, parent, hash_val, verdict, comment):
    """Calls the Create Verdict Change Request API."""
    new_verdict_enum = (
        self.messages.WildfireVerdictChangeRequest.NewVerdictValueValuesEnum(
            verdict.upper()
        )
    )
    verdict_change_request = self.messages.WildfireVerdictChangeRequest(
        sha256=hash_val,
        newVerdict=new_verdict_enum,
        comment=comment,
    )
    request = self.messages.NetworksecurityOrganizationsLocationsFirewallEndpointsWildfireVerdictChangeRequestsCreateRequest(
        parent=parent,
        wildfireVerdictChangeRequest=verdict_change_request,
    )
    return self._service.Create(request)

  def ListVerdictChangeRequests(
      self, parent, limit=None, page_size=None
  ):
    """Calls the List Verdict Change Requests API."""
    list_request = self.messages.NetworksecurityOrganizationsLocationsFirewallEndpointsWildfireVerdictChangeRequestsListRequest(
        parent=parent,
    )
    return list_pager.YieldFromList(
        self._service,
        list_request,
        batch_size=page_size,
        limit=limit,
        field='wildfireVerdictChangeRequests',
        batch_size_attribute='pageSize',
    )

  def GetOperationsRef(self, operation):
    """Operations to Resource used for `waiter.WaitFor`."""
    return self._resource_parser.ParseRelativeName(
        operation.name,
        ORG_OPERATIONS_COLLECTION,
        False,
        self.api_version,
    )

  def WaitForOperation(
      self,
      operation_ref,
      message,
      has_result=False,
      max_wait=datetime.timedelta(seconds=600),
  ):
    """Waits for an operation to complete."""
    if has_result:
      poller = waiter.CloudOperationPoller(
          self._service, self._operations_client
      )
    else:
      poller = waiter.CloudOperationPollerNoResources(self._operations_client)

    response = waiter.WaitFor(
        poller, operation_ref, message, max_wait_ms=max_wait.seconds * 1000
    )
    return response

  def GetVerdictChangeRequest(self, name):
    """Calls the Get Verdict Change Request API.

    Args:
      name: The resource name of the WildfireVerdictChangeRequest to retrieve.

    Returns:
      The WildfireVerdictChangeRequest resource.
    """
    request = self.messages.NetworksecurityOrganizationsLocationsFirewallEndpointsWildfireVerdictChangeRequestsGetRequest(
        name=name,
    )
    return self._service.Get(request)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Spanner instance API helper."""

import datetime
import re

from apitools.base.py import list_pager
from cloudsdk.google.protobuf import timestamp_pb2
from googlecloudsdk.api_lib.spanner import response_util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import console_io


# The list of pre-defined IAM roles in Spanner.
KNOWN_ROLES = [
    'roles/spanner.admin', 'roles/spanner.databaseAdmin',
    'roles/spanner.databaseReader', 'roles/spanner.databaseUser',
    'roles/spanner.viewer'
]

# Timeout to use in ListInstances for unreachable instances.
UNREACHABLE_INSTANCE_TIMEOUT = datetime.timedelta(seconds=20)

_SPANNER_API_NAME = 'spanner'
_SPANNER_API_VERSION = 'v1'

_FIELD_MASK_AUTOSCALING_CONFIG = 'autoscalingConfig'


def MaybeGetAutoscalingOverride(msgs, asymmetric_autoscaling_option):
  """Returns AutoscalingConfigOverrides object if any override is found in the parsed command-line flag key-value pairs, otherwise returns None."""
  if (
      'min_nodes' not in asymmetric_autoscaling_option
      and 'max_nodes' not in asymmetric_autoscaling_option
      and 'min_processing_units' not in asymmetric_autoscaling_option
      and 'max_processing_units' not in asymmetric_autoscaling_option
      and 'high_priority_cpu_target' not in asymmetric_autoscaling_option
      and 'total_cpu_target' not in asymmetric_autoscaling_option
      and 'disable_high_priority_cpu_autoscaling'
      not in asymmetric_autoscaling_option
      and 'disable_total_cpu_autoscaling' not in asymmetric_autoscaling_option
  ):
    return None

  obj = msgs.AutoscalingConfigOverrides(
      autoscalingLimits=msgs.AutoscalingLimits()
  )
  if 'min_nodes' in asymmetric_autoscaling_option:
    obj.autoscalingLimits.minNodes = asymmetric_autoscaling_option['min_nodes']
  if 'max_nodes' in asymmetric_autoscaling_option:
    obj.autoscalingLimits.maxNodes = asymmetric_autoscaling_option['max_nodes']
  if 'min_processing_units' in asymmetric_autoscaling_option:
    obj.autoscalingLimits.minProcessingUnits = asymmetric_autoscaling_option[
        'min_processing_units'
    ]
  if 'max_processing_units' in asymmetric_autoscaling_option:
    obj.autoscalingLimits.maxProcessingUnits = asymmetric_autoscaling_option[
        'max_processing_units'
    ]
  if 'high_priority_cpu_target' in asymmetric_autoscaling_option:
    obj.autoscalingTargetHighPriorityCpuUtilizationPercent = (
        asymmetric_autoscaling_option['high_priority_cpu_target']
    )
  if 'total_cpu_target' in asymmetric_autoscaling_option:
    obj.autoscalingTargetTotalCpuUtilizationPercent = (
        asymmetric_autoscaling_option['total_cpu_target']
    )
  if 'disable_high_priority_cpu_autoscaling' in asymmetric_autoscaling_option:
    obj.disableHighPriorityCpuAutoscaling = (
        asymmetric_autoscaling_option['disable_high_priority_cpu_autoscaling']
    )
  if 'disable_total_cpu_autoscaling' in asymmetric_autoscaling_option:
    obj.disableTotalCpuAutoscaling = (
        asymmetric_autoscaling_option['disable_total_cpu_autoscaling']
    )
  return obj


# Merges existing_overrides with new_overrides and returned the merged result.
def MergeAutoscalingConfigOverride(msgs, existing_overrides, new_overrides):
  """Merges two AutoscalingConfigOverrides objects.

  Args:
    msgs: The messages module for the Spanner API.
    existing_overrides: The existing AutoscalingConfigOverrides object.
    new_overrides: The new AutoscalingConfigOverrides object to merge.

  Returns:
    The merged AutoscalingConfigOverrides object.
  """
  if existing_overrides is None and new_overrides is None:
    return None

  if existing_overrides is None:
    return new_overrides

  if new_overrides is None:
    return existing_overrides

  # First, copy the existing values.
  result = existing_overrides
  # Next, assign any new limits overrides if any.
  if new_overrides.autoscalingLimits is not None:
    # Make sure autoscalingLimits is not None in the result.
    if result.autoscalingLimits is None:
      result.autoscalingLimits = msgs.AutoscalingLimits()
    if new_overrides.autoscalingLimits.minNodes is not None:
      result.autoscalingLimits.minNodes = (
          new_overrides.autoscalingLimits.minNodes
      )
    if new_overrides.autoscalingLimits.maxNodes is not None:
      result.autoscalingLimits.maxNodes = (
          new_overrides.autoscalingLimits.maxNodes
      )
    if new_overrides.autoscalingLimits.minProcessingUnits is not None:
      result.autoscalingLimits.minProcessingUnits = (
          new_overrides.autoscalingLimits.minProcessingUnits
      )
    if new_overrides.autoscalingLimits.maxProcessingUnits is not None:
      result.autoscalingLimits.maxProcessingUnits = (
          new_overrides.autoscalingLimits.maxProcessingUnits
      )

  # Finally, assign any target overrides if any.
  if (
      new_overrides.autoscalingTargetHighPriorityCpuUtilizationPercent
      is not None
  ):
    result.autoscalingTargetHighPriorityCpuUtilizationPercent = (
        new_overrides.autoscalingTargetHighPriorityCpuUtilizationPercent
    )

  if new_overrides.autoscalingTargetTotalCpuUtilizationPercent is not None:
    result.autoscalingTargetTotalCpuUtilizationPercent = (
        new_overrides.autoscalingTargetTotalCpuUtilizationPercent
    )

  if new_overrides.disableHighPriorityCpuAutoscaling is not None:
    result.disableHighPriorityCpuAutoscaling = (
        new_overrides.disableHighPriorityCpuAutoscaling
    )
    if result.disableHighPriorityCpuAutoscaling:
      result.autoscalingTargetHighPriorityCpuUtilizationPercent = None

  if new_overrides.disableTotalCpuAutoscaling is not None:
    result.disableTotalCpuAutoscaling = new_overrides.disableTotalCpuAutoscaling
    if result.disableTotalCpuAutoscaling:
      result.autoscalingTargetTotalCpuUtilizationPercent = None
  return result


# Set instance_obj.autoscalingConfig.asymmetricAutoscalingOptions by merging
# options found in the current_instance and patch requested, asym_option_patch.
def PatchAsymmetricAutoscalingOptions(
    msgs, instance_obj, current_instance, asym_options_patch
):
  """Patch asymmetric autoscaling options.

  Args:
    msgs: API messages module.
    instance_obj: The instance object to patch.
    current_instance: The current instance object.
    asym_options_patch: A list of AsymmetricAutoscalingOption objects to patch.
  """
  option_by_location = {}
  if config := current_instance.autoscalingConfig:
    for existing_option in config.asymmetricAutoscalingOptions:
      option_by_location[existing_option.replicaSelection.location] = (
          existing_option
      )

  for patch_option in asym_options_patch:
    location = patch_option.replicaSelection.location
    if location in option_by_location:
      # Update existing option
      existing_option = option_by_location[location]
      option_by_location[location].overrides = MergeAutoscalingConfigOverride(
          msgs, existing_option.overrides, patch_option.overrides
      )
    else:
      # Add new option
      option_by_location[location] = patch_option

  instance_obj.autoscalingConfig.asymmetricAutoscalingOptions.clear()
  for opt in option_by_location.values():
    instance_obj.autoscalingConfig.asymmetricAutoscalingOptions.append(opt)


def Create(
    instance,
    config,
    description,
    nodes,
    processing_units=None,
    autoscaling_min_nodes=None,
    autoscaling_max_nodes=None,
    autoscaling_min_processing_units=None,
    autoscaling_max_processing_units=None,
    autoscaling_high_priority_cpu_target=None,
    autoscaling_total_cpu_target=None,
    autoscaling_storage_target=None,
    asymmetric_autoscaling_options=None,
    disable_downscaling=None,
    instance_type=None,
    expire_behavior=None,
    ssd_cache=None,
    edition=None,
    default_backup_schedule_type=None,
    tags=None,
    labels=None,
):
  """Create a new instance.

  Args:
    instance: The instance to create.
    config: The instance config to use.
    description: The instance description.
    nodes: The number of nodes to use.
    processing_units: The number of processing units to use.
    autoscaling_min_nodes: The minimum number of nodes to use.
    autoscaling_max_nodes: The maximum number of nodes to use.
    autoscaling_min_processing_units: The minimum number of processing units to
      use.
    autoscaling_max_processing_units: The maximum number of processing units to
      use.
    autoscaling_high_priority_cpu_target: The high priority CPU target to use.
      Zero is a valid value.
    autoscaling_total_cpu_target: The total CPU target to use. Zero is a valid
      value.
    autoscaling_storage_target: The storage target to use.
    asymmetric_autoscaling_options: A list of ordered dict of key-value pairs
      representing the asymmetric autoscaling options.
    disable_downscaling: Whether to disable downscaling for the instance.
    instance_type: The instance type to use.
    expire_behavior: The expire behavior to use.
    ssd_cache: The ssd cache to use.
    edition: The edition to use.
    default_backup_schedule_type: The type of default backup schedule to use.
    tags: The parsed tags value.
    labels: The parsed labels value.

  Returns:
    The created instance.
  """
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  # Module containing the definitions of messages for the specified API.
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  config_ref = resources.REGISTRY.Parse(
      config,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instanceConfigs',
  )
  project_ref = resources.REGISTRY.Create(
      'spanner.projects', projectsId=properties.VALUES.core.project.GetOrFail
  )
  instance_obj = msgs.Instance(
      config=config_ref.RelativeName(), displayName=description
  )
  if nodes:
    instance_obj.nodeCount = nodes
  elif processing_units:
    instance_obj.processingUnits = processing_units
  elif (
      autoscaling_min_nodes
      or autoscaling_max_nodes
      or autoscaling_min_processing_units
      or autoscaling_max_processing_units
      or autoscaling_high_priority_cpu_target is not None
      or autoscaling_total_cpu_target is not None
      or autoscaling_storage_target
      or disable_downscaling is not None
  ):
    instance_obj.autoscalingConfig = msgs.AutoscalingConfig(
        autoscalingLimits=msgs.AutoscalingLimits(
            minNodes=autoscaling_min_nodes,
            maxNodes=autoscaling_max_nodes,
            minProcessingUnits=autoscaling_min_processing_units,
            maxProcessingUnits=autoscaling_max_processing_units,
        ),
        autoscalingTargets=msgs.AutoscalingTargets(
            highPriorityCpuUtilizationPercent=autoscaling_high_priority_cpu_target,
            totalCpuUtilizationPercent=autoscaling_total_cpu_target,
            storageUtilizationPercent=autoscaling_storage_target,
        ),
        disableDownscaling=disable_downscaling,
    )
  if instance_type is not None:
    instance_obj.instanceType = instance_type
  if expire_behavior is not None:
    instance_obj.freeInstanceMetadata = msgs.FreeInstanceMetadata(
        expireBehavior=expire_behavior
    )
  if ssd_cache and ssd_cache.strip():
    instance_obj.ssdCache = (
        config_ref.RelativeName() + '/ssdCaches/' + ssd_cache.strip()
    )
  if edition is not None:
    instance_obj.edition = msgs.Instance.EditionValueValuesEnum(edition)
  if default_backup_schedule_type is not None:
    instance_obj.defaultBackupScheduleType = (
        msgs.Instance.DefaultBackupScheduleTypeValueValuesEnum(
            default_backup_schedule_type
        )
    )
  if tags is not None:
    instance_obj.tags = msgs.Instance.TagsValue(
        additionalProperties=[
            msgs.Instance.TagsValue.AdditionalProperty(key=key, value=value)
            for key, value in sorted(tags.items())
        ]
    )
  if labels is not None:
    instance_obj.labels = msgs.Instance.LabelsValue(
        additionalProperties=[
            msgs.Instance.LabelsValue.AdditionalProperty(key=key, value=value)
            for key, value in sorted(labels.items())
        ]
    )

  # Add asymmetric autoscaling options, if present.
  if asymmetric_autoscaling_options is not None:
    for asym_option in asymmetric_autoscaling_options:
      instance_obj.autoscalingConfig.asymmetricAutoscalingOptions.append(
          msgs.AsymmetricAutoscalingOption(
              overrides=MaybeGetAutoscalingOverride(msgs, asym_option),
              replicaSelection=msgs.InstanceReplicaSelection(
                  location=asym_option['location']
              ),
          )
      )
  req = msgs.SpannerProjectsInstancesCreateRequest(
      parent=project_ref.RelativeName(),
      createInstanceRequest=msgs.CreateInstanceRequest(
          instanceId=instance, instance=instance_obj
      ),
  )
  return client.projects_instances.Create(req)


def SetPolicy(instance_ref, policy, field_mask=None):
  """Saves the given policy on the instance, overwriting whatever exists."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  policy.version = iam_util.MAX_LIBRARY_IAM_SUPPORTED_VERSION
  req = msgs.SpannerProjectsInstancesSetIamPolicyRequest(
      resource=instance_ref.RelativeName(),
      setIamPolicyRequest=msgs.SetIamPolicyRequest(policy=policy,
                                                   updateMask=field_mask))
  return client.projects_instances.SetIamPolicy(req)


def GetIamPolicy(instance_ref):
  """Gets the IAM policy on an instance."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  req = msgs.SpannerProjectsInstancesGetIamPolicyRequest(
      resource=instance_ref.RelativeName(),
      getIamPolicyRequest=msgs.GetIamPolicyRequest(
          options=msgs.GetPolicyOptions(
              requestedPolicyVersion=
              iam_util.MAX_LIBRARY_IAM_SUPPORTED_VERSION)))
  return client.projects_instances.GetIamPolicy(req)


def Delete(instance):
  """Delete an instance."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  ref = resources.REGISTRY.Parse(
      instance,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instances')
  req = msgs.SpannerProjectsInstancesDeleteRequest(name=ref.RelativeName())
  return client.projects_instances.Delete(req)


def Get(instance):
  """Get an instance by name."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  ref = resources.REGISTRY.Parse(
      instance,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instances')
  req = msgs.SpannerProjectsInstancesGetRequest(name=ref.RelativeName())
  return client.projects_instances.Get(req)


def List():
  """List instances in the project."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  project_ref = resources.REGISTRY.Create(
      'spanner.projects', projectsId=properties.VALUES.core.project.GetOrFail)
  tp_proto = timestamp_pb2.Timestamp()
  tp_proto.FromDatetime(
      datetime.datetime.utcnow() + UNREACHABLE_INSTANCE_TIMEOUT)
  req = msgs.SpannerProjectsInstancesListRequest(
      parent=project_ref.RelativeName(),
      instanceDeadline=tp_proto.ToJsonString())
  return list_pager.YieldFromList(
      client.projects_instances,
      req,
      field='instances',
      batch_size_attribute='pageSize',
      get_field_func=response_util.GetFieldAndLogUnreachable)


def Patch(
    instance,
    description=None,
    nodes=None,
    processing_units=None,
    autoscaling_min_nodes=None,
    autoscaling_max_nodes=None,
    autoscaling_min_processing_units=None,
    autoscaling_max_processing_units=None,
    autoscaling_high_priority_cpu_target=None,
    autoscaling_total_cpu_target=None,
    autoscaling_storage_target=None,
    asymmetric_autoscaling_options=None,
    disable_downscaling=None,
    clear_asymmetric_autoscaling_options=None,
    instance_type=None,
    expire_behavior=None,
    ssd_cache_id=None,
    edition=None,
    default_backup_schedule_type=None,
    args=None,
):
  """Update an instance."""
  fields = []
  if description is not None:
    fields.append('displayName')
  if nodes is not None:
    fields.append('nodeCount,autoscalingConfig')
  if processing_units is not None:
    fields.append('processingUnits,autoscalingConfig')

  if (
      (autoscaling_min_nodes and autoscaling_max_nodes)
      or (autoscaling_min_processing_units and autoscaling_max_processing_units)
  ) and ((autoscaling_high_priority_cpu_target is not None or
          autoscaling_total_cpu_target is not None)
         and autoscaling_storage_target):
    fields.append(_FIELD_MASK_AUTOSCALING_CONFIG)
  else:
    if autoscaling_min_nodes:
      fields.append('autoscalingConfig.autoscalingLimits.minNodes')
    if autoscaling_max_nodes:
      fields.append('autoscalingConfig.autoscalingLimits.maxNodes')
    if autoscaling_min_processing_units:
      fields.append('autoscalingConfig.autoscalingLimits.minProcessingUnits')
    if autoscaling_max_processing_units:
      fields.append('autoscalingConfig.autoscalingLimits.maxProcessingUnits')
    if autoscaling_high_priority_cpu_target is not None:
      fields.append(
          'autoscalingConfig.autoscalingTargets.highPriorityCpuUtilizationPercent'
      )
    if autoscaling_total_cpu_target is not None:
      fields.append(
          'autoscalingConfig.autoscalingTargets.totalCpuUtilizationPercent'
      )
    if autoscaling_storage_target:
      fields.append(
          'autoscalingConfig.autoscalingTargets.storageUtilizationPercent'
      )
    if disable_downscaling is not None:
      fields.append('autoscalingConfig.disableDownscaling')
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)

  instance_obj = msgs.Instance(displayName=description)
  if processing_units:
    instance_obj.processingUnits = processing_units
  elif nodes:
    instance_obj.nodeCount = nodes
  elif (
      autoscaling_min_nodes
      or autoscaling_max_nodes
      or autoscaling_min_processing_units
      or autoscaling_max_processing_units
      or autoscaling_high_priority_cpu_target is not None
      or autoscaling_total_cpu_target is not None
      or autoscaling_storage_target
      or disable_downscaling is not None
  ):
    instance_obj.autoscalingConfig = msgs.AutoscalingConfig(
        autoscalingLimits=msgs.AutoscalingLimits(
            minNodes=autoscaling_min_nodes,
            maxNodes=autoscaling_max_nodes,
            minProcessingUnits=autoscaling_min_processing_units,
            maxProcessingUnits=autoscaling_max_processing_units,
        ),
        autoscalingTargets=msgs.AutoscalingTargets(
            highPriorityCpuUtilizationPercent=autoscaling_high_priority_cpu_target,
            totalCpuUtilizationPercent=autoscaling_total_cpu_target,
            storageUtilizationPercent=autoscaling_storage_target,
        ),
        disableDownscaling=disable_downscaling,
    )

  if asymmetric_autoscaling_options is not None:
    if _FIELD_MASK_AUTOSCALING_CONFIG not in fields:
      fields.append('autoscalingConfig.asymmetricAutoscalingOptions')
    current_instance = Get(instance)
    asym_options_patch = []
    # Create AsymmetricAutoscalingOption objects from the flag value (key-value
    # pairs).
    for asym_option in asymmetric_autoscaling_options:
      asym_options_patch.append(
          msgs.AsymmetricAutoscalingOption(
              replicaSelection=msgs.InstanceReplicaSelection(
                  location=asym_option['location']
              ),
              overrides=MaybeGetAutoscalingOverride(msgs, asym_option),
          )
      )
    if instance_obj.autoscalingConfig is None:
      instance_obj.autoscalingConfig = msgs.AutoscalingConfig()
    PatchAsymmetricAutoscalingOptions(
        msgs, instance_obj, current_instance, asym_options_patch
    )

  if clear_asymmetric_autoscaling_options is not None:
    if _FIELD_MASK_AUTOSCALING_CONFIG not in fields:
      fields.append('autoscalingConfig.asymmetricAutoscalingOptions')
    current_instance = Get(instance)
    locations_to_remove = set(clear_asymmetric_autoscaling_options)
    if instance_obj.autoscalingConfig is None:
      instance_obj.autoscalingConfig = msgs.AutoscalingConfig()
    instance_obj.autoscalingConfig.asymmetricAutoscalingOptions = []
    for (
        asym_option
    ) in current_instance.autoscalingConfig.asymmetricAutoscalingOptions:
      if asym_option.replicaSelection.location not in locations_to_remove:
        instance_obj.autoscalingConfig.asymmetricAutoscalingOptions.append(
            asym_option
        )

  if instance_type is not None:
    fields.append('instanceType')
    instance_obj.instanceType = instance_type
  if expire_behavior is not None:
    fields.append('freeInstanceMetadata.expireBehavior')
    instance_obj.freeInstanceMetadata = msgs.FreeInstanceMetadata(
        expireBehavior=expire_behavior)

  if ssd_cache_id is not None:
    fields.append('ssdCache')
    ssd_cache = ''
    if ssd_cache_id.strip():
      instance_res = Get(instance)
      ssd_cache = instance_res.config + '/ssdCaches/' + ssd_cache_id.strip()
    instance_obj.ssdCache = ssd_cache
  if edition is not None:
    fields.append('edition')
    instance_obj.edition = msgs.Instance.EditionValueValuesEnum(edition)
  if default_backup_schedule_type is not None:
    fields.append('defaultBackupScheduleType')
    instance_obj.defaultBackupScheduleType = (
        msgs.Instance.DefaultBackupScheduleTypeValueValuesEnum(
            default_backup_schedule_type
        )
    )

  if args is not None:

    def GetLabels():
      return Get(instance).labels

    labels_update = labels_util.ProcessUpdateArgsLazy(
        args, msgs.Instance.LabelsValue, GetLabels
    )
    if labels_update.needs_update:
      instance_obj.labels = labels_update.labels
      fields.append('labels')

  ref = resources.REGISTRY.Parse(
      instance,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instances')
  req = msgs.SpannerProjectsInstancesPatchRequest(
      name=ref.RelativeName(),
      updateInstanceRequest=msgs.UpdateInstanceRequest(
          fieldMask=','.join(fields), instance=instance_obj))
  return client.projects_instances.Patch(req)


def GetLocations(instance, verbose_flag):
  """Get all the replica regions for an instance."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  instance_res = Get(instance)
  config_req = msgs.SpannerProjectsInstanceConfigsGetRequest(
      name=instance_res.config)
  config_res = client.projects_instanceConfigs.Get(config_req)
  if verbose_flag:
    command_output = []
    for item in config_res.replicas:
      command_output.append({'location': item.location, 'type': item.type})
  else:
    region_set = set()
    for item in config_res.replicas:
      region_set.add(item.location)
    command_output = [{'location': item} for item in region_set]
  return command_output


def Move(instance, target_instance_config, target_database_move_configs):
  """Moves an instance from one instance-config to another.

  Args:
      instance: Instance to move.
      target_instance_config: The target instance configuration to move the
        instance.
      target_database_move_configs: Configurations for databases in the
        destination instance config.

  The configs can be google-managed or user-managed.
  Ex: gcloud spanner instances move instance-to-move
  --target-config=instance-config-to-move-to

  Above example will move the instance(instance-to-move) to the following
  instance config(instance-config-to-move-to).
  """
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  config_ref = resources.REGISTRY.Parse(
      target_instance_config,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instanceConfigs',
  )
  instance_ref = resources.REGISTRY.Parse(
      instance,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instances',
  )
  console_io.PromptContinue(
      message=(
          'You are about to move instance {0} from {1} to {2}. This is a'
          ' long-running operation with potential service'
          ' implications:\n\n\n\t* Increased latencies: Read and write'
          ' operations may experience delays.\n\n\t* Elevated abort rate:'
          ' Transactions may have a higher chance of failing.\n\n\t* Spiked CPU'
          ' utilization: System resources will be strained, impacting'
          ' performance.\n\n\t* Additional costs: Instance moves incur extra'
          ' charges, as described in the documentation.\n\n\t* Backups: It is'
          ' important that you copy your backups before moving your instance.'
          ' Backups need to be deleted from the Instance before the move. You'
          ' cannot create a backup while the move is in progress.\n\nBefore'
          ' proceeding, and for detailed information and best practices, refer'
          ' to the documentation at'
          ' https://cloud.google.com/spanner/docs/move-instance#move-prerequisites.'
          .format(
              instance, GetInstanceConfig(instance), target_instance_config
          )
      ),
      cancel_on_no=True,
      prompt_string='Do you want to proceed',
  )
  req_args = {'targetConfig': config_ref.RelativeName()}
  if target_database_move_configs is not None:
    req_args['targetDatabaseMoveConfigs'] = []
    for target_database_move_config in target_database_move_configs:
      kms_key_names = target_database_move_config['kms-key-names'].split(',')
      encryption_config_args = {}
      encryption_config_args['kmsKeyNames'] = []
      for kms_key_name in kms_key_names:
        encryption_config_args['kmsKeyNames'].append(kms_key_name)
      encryption_config = msgs.InstanceEncryptionConfig(
          **encryption_config_args
      )
      req_args['targetDatabaseMoveConfigs'].append(
          msgs.DatabaseMoveConfig(
              databaseId=target_database_move_config['database-id'],
              encryptionConfig=encryption_config,
          )
      )
  move_req = msgs.SpannerProjectsInstancesMoveRequest(
      moveInstanceRequest=msgs.MoveInstanceRequest(**req_args),
      name=instance_ref.RelativeName(),
  )
  move_operation_id = client.projects_instances.Move(move_req).name
  operation_id = re.search('.*/operations/(.*)', move_operation_id).group(1)
  print(
      '\nInstance move started for {0}\n\n'
      'Track progress with: gcloud spanner operations'
      ' describe {1} --instance={2}'.format(
          instance_ref.RelativeName(), operation_id, instance
      )
  )


def GetInstanceConfig(instance):
  """Get the instance config of the passed instance."""
  client = apis.GetClientInstance(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  msgs = apis.GetMessagesModule(_SPANNER_API_NAME, _SPANNER_API_VERSION)
  instance_ref = resources.REGISTRY.Parse(
      instance,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='spanner.projects.instances',
  )
  instance_req = msgs.SpannerProjectsInstancesGetRequest(
      name=instance_ref.RelativeName(), fieldMask='config'
  )
  instance_info = client.projects_instances.Get(instance_req)
  instance_config = re.search(
      '.*/instanceConfigs/(.*)', instance_info.config
  ).group(1)
  return instance_config



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Backup and DR Management Servers client."""


from googlecloudsdk.api_lib.backupdr import util


class ManagementServersClient(util.BackupDrClientBase):
  """Cloud Backup and DR Management client."""

  def __init__(self, api_version=util.DEFAULT_API_VERSION):
    super(ManagementServersClient, self).__init__(api_version=api_version)
    self.service = self.client.projects_locations_managementServers

  def Create(self, resource, network=None):
    parent = resource.Parent().RelativeName()
    management_server_id = resource.Name()
    if network is not None:
      networks = [self.messages.NetworkConfig(network=network)]
      management_server = self.messages.ManagementServer(
          networks=networks,
          type=self.messages.ManagementServer.TypeValueValuesEnum.BACKUP_RESTORE,
      )
    else:
      management_server = self.messages.ManagementServer(
          type=self.messages.ManagementServer.TypeValueValuesEnum.BACKUP_RESTORE,
      )

    request = (
        self.messages.BackupdrProjectsLocationsManagementServersCreateRequest(
            parent=parent,
            managementServer=management_server,
            managementServerId=management_server_id,
        )
    )
    return self.service.Create(request)

  def Delete(self, resource):
    request = (
        self.messages.BackupdrProjectsLocationsManagementServersDeleteRequest(
            name=resource.RelativeName()
        )
    )
    return self.service.Delete(request)
